<?php

/*
	dbinit - initialise the database or repair an existing database
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

// this file contains the complete definition of the database structure,
// and is enough to create the database from scratch

class dbinit
{
	function dbinit(&$db)
	{
		$this->db = &$db;
		$this->prefix = $this->db->get_prefix();
	}

	function preparetables($errorcallback = null)
		// this actually sets up the database tables
	{
		require_once(BLUESTONE_DIR . 'dbprepare.inc.php');

		$dbprepare = new dbprepare($this->db, 'MyISAM', 'utf8_general_ci');

		if ($errorcallback) $dbprepare->seterrorcallback($errorcallback);

		$dbprepare->suppressbigchanges(false);
		$dbprepare->preparemultiple($this->gettabledefs());
	}

	private static function getenumfield($fieldname, $isnull = false)
		// gets values for enum fields from the configuration class
	{
		$fields = array_keys(nicnames_config::$resourcetypes[$fieldname]);
		array_map('addslashes', $fields);
		$column = array('enum(\'' . implode('\',\'', $fields) . '\')');
		if ($isnull)
		{
			$column[] = NULL;
			$column[] = true;
		}
		return $column;
	}

	function gettabledefs()
		// this contains the definition of the database structure
	{
		$tabledefs = array();

		/* classification */

		$tabledefs['classification']['fields'] = array(
			'classification_ID'         => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'classification_identityID' => array('int(11)'),
			'classification_code'       => array('varchar(255)', NULL, true),
			'classification_text'       => array('varchar(255)', NULL, true),
			'classification_type'       => self::getenumfield('classification_type'),
			'classification_startyear'  => array('int(4)', NULL, true),
			'classification_endyear'    => array('int(4)', NULL, true),
			'classification_comment'    => array('text', NULL, true),
			'classification_incidence'  => array('int(11)', '1'),
			'classification_score'      => array('float', '1'),
			'classification_verified'   => array('tinyint(1)', '0'),
		);
		$tabledefs['classification']['indexes'] = array(
			'PRIMARY' => array('PRIMARY', 'classification_ID'),
			'classification_identityID' => array('KEY', 'classification_identityID'),
		);

		/* event */

		$tabledefs['event']['fields'] = array(
			'event_ID'         => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'event_identityID' => array('int(11)'),
			'event_title'      => array('varchar(255)'),
			'event_type'       => self::getenumfield('event_type'),
			'event_reltype'    => self::getenumfield('event_reltype'),
			'event_startyear'  => array('int(4)', NULL, true),
			'event_endyear'    => array('int(4)', NULL, true),
			'event_comment'    => array('text', NULL, true),
			'event_incidence'  => array('int(11)', '1'),
			'event_score'      => array('float', '1'),
			'event_verified'   => array('tinyint(1)', '0'),
		);
		$tabledefs['event']['indexes'] = array(
			'PRIMARY'                  => array('PRIMARY', 'event_ID'),
			'relation_identityID' => array('KEY', 'event_identityID'),
		);

		/* identifier */

		$tabledefs['identifier']['fields'] = array(
			'identifier_ID'         => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'identifier_identityID' => array('int(11)'),
			'identifier_identifier' => array('varchar(255)'),
			'identifier_type'       => self::getenumfield('identifier_type'),
			'identifier_href'       => array('varchar(255)', NULL, true),
			'identifier_startyear'  => array('int(4)', NULL, true),
			'identifier_endyear'    => array('int(4)', NULL, true),
			'identifier_comment'    => array('text', NULL, true),
			'identifier_incidence'  => array('int(11)', '1'),
			'identifier_score'      => array('float', '1'),
			'identifier_verified'   => array('tinyint(1)', '0'),
		);
		$tabledefs['identifier']['indexes'] = array(
			'PRIMARY' => array('PRIMARY', 'identifier_ID'),
			'identifier_identityID' => array('KEY', 'identifier_identityID'),
		);

		/* identity */

		$tabledefs['identity']['fields'] = array(
			'identity_ID'               => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'identity_birthyear'        => array('int(4)', NULL, true),
			'identity_comment'          => array('text', NULL, true),
			'identity_createddate'      => array('int(11)'),
			'identity_lastmodifieddate' => array('int(11)'),
			'identity_affiliation'      => array('enum(\'affiliated\',\'previouslyaffiliated\',\'none\')', 'none'), 
			'identity_status'           => array('enum(\'normal\',\'nomatch\',\'uncertainmatch\',\'requestedverification\')', 'normal'), 
		);
		$tabledefs['identity']['indexes'] = array(
			'PRIMARY'              => array('PRIMARY', 'identity_ID'),
			'identity_status'      => array('KEY', 'identity_status,identity_lastmodifieddate'),
			'identity_affiliation' => array('KEY', 'identity_affiliation'),
			'identity_lastmodifieddate' => array('KEY', 'identity_lastmodifieddate'),
		);

		/* identity_alias */

		$tabledefs['identity_alias']['fields'] = array(
			'identity_alias_ID'         => array('int(11)'),
			'identity_alias_resolution' => array('enum(\'merged\',\'deleted\')'),
			'identity_alias_refID'      => array('int(11)', null, true),
			'identity_alias_createddate' => array('int(11)'),
			'identity_alias_lastmodifieddate' => array('int(11)'),
			);
		$tabledefs['identity_alias']['indexes'] = array(
			'PRIMARY'             => array('PRIMARY', 'identity_alias_ID'),
			'identity_alias_refID' => array('KEY', 'identity_alias_refID'),
			'identity_alias_lastmodifieddate' => array('KEY', 'identity_alias_lastmodifieddate'),
			'identity_alias_resolution' => array('KEY', 'identity_alias_resolution'),
			);

		/* index */

		$tabledefs['index']['fields'] = array(
			'index_wordID'       => array('int(11)'),
			'index_identityID'   => array('int(11)'),
			'index_category'     => array('enum(\'identity\',\'name\',\'organisation\',\'event\',\'identifier\',\'resource\',\'classification\',\'person\')'),
			'index_score'        => array('tinyint(4)'),
		);
		$tabledefs['index']['indexes'] = array(
			'PRIMARY'            => array('PRIMARY', 'index_identityID,index_wordID,index_category'),
			'index_wordID'       => array('KEY', 'index_wordID'),
		);

		/* index_stem */

		$tabledefs['index_stem']['fields'] = array(
			'index_stem_wordID'     => array('int(11)'),
			'index_stem_stemwordID' => array('int(11)'),
		);
		$tabledefs['index_stem']['indexes'] = array(
			'PRIMARY'               => array('PRIMARY', 'index_stem_wordID'),
			'index_stem_stemwordID' => array('KEY', 'index_stem_stemwordID'),
		);

		/* index_word */

		$tabledefs['index_word']['fields'] = array(
			'index_word_ID'     => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'index_word_text'   => array('varchar(63)'),
			'index_word_phon'   => array('int(11)'),
			'index_word_rphon'  => array('int(11)'),
			'index_word_freq'   => array('int(11)'),
		);
		$tabledefs['index_word']['indexes'] = array(
			'PRIMARY'           => array('PRIMARY', 'index_word_ID'),
			'index_word_text'   => array('UNIQUE', 'index_word_text'),
			'index_word_phon'   => array('KEY', 'index_word_phon'),
			'index_word_rphon'  => array('KEY', 'index_word_rphon'),
		);

		/* name */

		$tabledefs['name']['fields'] = array(
			'name_ID'                 => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'name_identityID'         => array('int(11)'),
			'name_surname'            => array('varchar(127)'),
			'name_givennames'         => array('varchar(255)', NULL, true),
			'name_title'              => array('varchar(127)', NULL, true),
			'name_surnamefirst'       => array('tinyint(1)', '0'),
			'name_type'               => self::getenumfield('name_type'),
			'name_startyear'          => array('int(11)', NULL, true),
			'name_endyear'            => array('int(11)', NULL, true),
			'name_comment'            => array('text', NULL, true),
			'name_incidence'          => array('int(11)', '1'),
			'name_score'              => array('float', '1'),
			'name_verified'           => array('tinyint(1)', '0'),
		);
		$tabledefs['name']['indexes'] = array(
			'PRIMARY'                 => array('PRIMARY', 'name_ID'),
			'name_identityID'         => array('KEY', 'name_identityID'),
		);

		/* organisation */

		$tabledefs['organisation']['fields'] = array(
			'organisation_ID'         => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'organisation_identityID' => array('int(11)'),
			'organisation_title'      => array('varchar(255)'),
			'organisation_type'       => self::getenumfield('organisation_type'),
			'organisation_reltype'    => self::getenumfield('organisation_reltype'),
			'organisation_startyear'  => array('int(4)', NULL, true),
			'organisation_endyear'    => array('int(4)', NULL, true),
			'organisation_comment'    => array('text', NULL, true),
			'organisation_incidence'  => array('int(11)', '1'),
			'organisation_score'      => array('float', '1'),
			'organisation_verified'   => array('tinyint(1)', '0'),
		);
		$tabledefs['organisation']['indexes'] = array(
			'PRIMARY'                => array('PRIMARY', 'organisation_ID'),
			'organisation_identityID' => array('KEY', 'organisation_identityID'),
		);

		/* person */

		$tabledefs['person']['fields'] = array(
			'person_ID'               => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'person_identityID'       => array('int(11)'),
			'person_secondidentityID' => array('int(11)'),
			'person_type'             => self::getenumfield('person_type'),
			'person_startyear'        => array('int(4)', NULL, true),
			'person_endyear'          => array('int(4)', NULL, true),
			'person_comment'          => array('text', NULL, true),
			'person_incidence'        => array('int(11)', '1'),
			'person_score'            => array('float', '1'),
			'person_verified'         => array('tinyint(1)', '0'),
		);
		$tabledefs['person']['indexes'] = array(
			'PRIMARY'                 => array('PRIMARY', 'person_ID'),
			'person_identityID'       => array('KEY', 'person_identityID'),
			'person_secondidentityID' => array('KEY', 'person_secondidentityID'),
		);

		/* resource */

		$tabledefs['resource']['fields'] = array(
			'resource_ID'         => array('int(11)', NULL, false, 'AUTO_INCREMENT'),
			'resource_identityID' => array('int(11)'),
			'resource_title'      => array('varchar(255)'),
			'resource_type'       => self::getenumfield('resource_type'),
			'resource_reltype'    => self::getenumfield('resource_reltype'),
			'resource_startyear'  => array('int(4)', NULL, true),
			'resource_endyear'    => array('int(4)', NULL, true),
			'resource_comment'    => array('text', NULL, true),
			'resource_incidence'  => array('int(11)', '1'),
			'resource_score'      => array('float', '1'),
			'resource_verified'   => array('tinyint(1)', '0'),
		);
		$tabledefs['resource']['indexes'] = array(
			'PRIMARY'                     => array('PRIMARY', 'resource_ID'),
			'resource_identityID' => array('KEY', 'resource_identityID'),
		);

		return $tabledefs;

	}

}

?>
