<?php

/*
	diagnostics - for checking an installation and performing pre/post install tasks
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

class diagnostics
{
	function diagnostics(&$db)
	{
		$this->db = &$db;
		$this->prefix = $this->db->get_prefix();
	}

	function seterror($callback, $notice, $type = 'notice')
	{
		if ($callback === null) return;
		call_user_func($callback, $notice, $type);
	}

	function preinstall($errorcallback = null)
		// run any pre-installation/upgrade checks and setup tasks
	{
		$this->seterror($errorcallback, 'Running pre-install checks...', 'notice');

		// check database connection
		if ($this->db->is_connected())
			$this->seterror($errorcallback, 'Database connection is OK.');
		else
		{
			$this->seterror($errorcallback, 'Database connection failed.  Please check the database connection settings in the configuration file (config.inc.php) to ensure the correct hostname, login and database name are specified.  The server must be configured to accept connections from this host.', 'error');
			return false;
		}

		return true;
	}


	function postinstall($errorcallback = null)
		// run any post-installation upgrade checks and tasks
	{
		$this->seterror($errorcallback, 'Running post-install diagnostics...', 'notice');

		// remove self-referential rows from person
		$this->db->query("DELETE FROM {$this->prefix}person WHERE
			person_identityID=person_secondidentityID");
		if ($rows = $this->db->affected_rows())
			$this->seterror($errorcallback, "Removed $rows erroneous self-referential people reference(s).", 'changed');

		// remove orphan rows from index
		$this->db->query("
			DELETE
				`{$this->prefix}index`
			FROM
				`{$this->prefix}index`
				LEFT JOIN {$this->prefix}identity ON
					identity_ID=index_identityID
			WHERE
				identity_ID IS NULL
			");
		if ($rows = $this->db->affected_rows())
			$this->seterror($errorcallback, "Removed $rows erroneous orphan rows from index table.", 'changed');

		// modify status values to new standard 20091112
		$this->db->query("UPDATE {$this->prefix}identity
			SET identity_status='nomatch'
			WHERE identity_status IN ('creatednomatch', 'createduncertain')");
		if ($rows = $this->db->affected_rows())
			$this->seterror($errorcallback, "Updated $rows identities to use new status type 'nomatch'.", 'changed');
		$this->db->query("UPDATE {$this->prefix}identity
			SET identity_status='uncertainmatch'
			WHERE identity_status IN ('updateduncertain', 'updatedmultimatch')");
		if ($rows = $this->db->affected_rows())
			$this->seterror($errorcallback, "Updated $rows identities to use new status type 'uncertainmatch'.", 'changed');

		return true;
	}

}

?>
