<?php

/*
	fetchidentities.inc.php - for fetching identities from the database
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/identity.inc.php');
require_once(RESOURCE_DIR . 'includes/identity_alias.inc.php');

class fetchidentities
{
	function fetchidentities(&$db)
	{
		$this->db = &$db;
		$this->prefix = $db->get_prefix();
	}

	function isidentity($identityid)
	// determines if an identity with the given ID exists.  This doesn't load the
	// whole identity from the database, so it is a faster alternative to
		// getidentities when you have only one ID and you don't need the identity object
	{
		$identityid = (int)$identityid;
		$result = $this->db->query_single("
			SELECT identity_ID FROM {$this->prefix}identity
			WHERE identity_ID=$identityid
			");
		return $result ? true : false;
	}

	function getidentities($ids, $recursionlimit = 1)
		// gets given identities from DB
		// ids may be a single identity ID, or an array of ids.  this function will
		// return an array of identity objects
		// note that this is recursive, in that related people (eg co-authors) will
		// be fetched, then their related people, etc.  The default recursion limit
		// is 1, ie only one level of related people are wanted.  This can be zero.
	{
		if (empty($ids)) return array();
		if (!is_array($ids)) $ids = array($ids);

		$idstext = $this->getidstring($ids);
		$identities = array();

		// first look for existing identities
		$this->db->query("
			SELECT * FROM {$this->prefix}identity WHERE identity_ID $idstext");

		while ($row = $this->db->fetch_array())
		{
			$identity = new identity();
			$identity->id = $row['identity_ID'];
			$identity->birthyear = $row['identity_birthyear'];
			$identity->comment = $row['identity_comment'];
			$identity->createddate = $row['identity_createddate'];
			$identity->lastmodifieddate = $row['identity_lastmodifieddate'];
			$identity->affiliation = $row['identity_affiliation'];
			$identity->status = $row['identity_status'];
			$identities[$identity->id] = $identity;
		}
		$this->db->free_result();

		// now look for aliased (ie deleted, merged) identities
		$this->db->query("
			SELECT * FROM {$this->prefix}identity_alias WHERE identity_alias_ID $idstext");

		while ($row = $this->db->fetch_array())
		{
			$alias = new identity_alias();
			$alias->id = $row['identity_alias_ID'];
			$alias->resolution = $row['identity_alias_resolution'];
			$alias->refid = $row['identity_alias_refID'];
			$alias->createddate = $row['identity_alias_createddate'];
			$alias->lastmodifieddate = $row['identity_alias_lastmodifieddate'];
			$identities[$alias->id] = $alias;
		}
		$this->db->free_result();

		if (empty($identities)) return array();

		$classifications = $this->getclassifications($ids);
		foreach ($classifications as $id => $val) $identities[$id]->classifications = $val;

		$events = $this->getevents($ids);
		foreach ($events as $id => $val) $identities[$id]->events = $val;

		$identifiers = $this->getidentifiers($ids);
		foreach ($identifiers as $id => $val) $identities[$id]->identifiers = $val;

		$names = $this->getnames($ids);
		foreach ($names as $id => $val) $identities[$id]->names = $val;

		$organisations = $this->getorganisations($ids);
		foreach ($organisations as $id => $val) $identities[$id]->organisations = $val;

		if ($recursionlimit > 0)
		{
			$persons = $this->getpersons($ids, $recursionlimit);
			foreach ($persons as $id => $val) $identities[$id]->persons = $val;
		}

		$resources = $this->getresources($ids);
		foreach ($resources as $id => $val) $identities[$id]->resources = $val;

		$ordered = array();
		foreach ($ids as $id)
			if (isset($identities[$id]) && isset($identities[$id]->id) && !isset($ordered[$id]))
				// the second isset is necessary for tolerating orphans
				$ordered[$id] = $identities[$id];

		return $ordered;
	}

	function getclassifications($ids)
	{
		$idstext = $this->getidstring($ids);

		$this->db->query("
			SELECT * FROM {$this->prefix}classification WHERE classification_identityID $idstext");

		$classifications = array();
		while ($row = $this->db->fetch_array())
		{
			$classification = new classification();
			$classification->id = $row['classification_ID'];
			$classification->code = $row['classification_code'];
			$classification->text = $row['classification_text'];
			$classification->type = $row['classification_type'];
			$this->populatecommon($classification, $row, 'classification_');
			$classifications[$row['classification_identityID']][] = $classification;
		}
		$this->db->free_result();

		return $classifications;
	}

	function getevents($ids)
	{
		$idstext = $this->getidstring($ids);

		$this->db->query("
			SELECT * FROM {$this->prefix}event WHERE event_identityID $idstext");

		$events = array();
		while ($row = $this->db->fetch_array())
		{
			$event = new event();
			$event->id = $row['event_ID'];
			$event->title = $row['event_title'];
			$event->type = $row['event_type'];
			$event->reltype = $row['event_reltype'];
			$this->populatecommon($event, $row, 'event_');
			$events[$row['event_identityID']][] = $event;
		}
		$this->db->free_result();

		return $events;
	}

	function getidentifiers($ids)
	{
		$idstext = $this->getidstring($ids);

		$this->db->query("
			SELECT * FROM {$this->prefix}identifier WHERE identifier_identityID $idstext");

		$identifiers = array();
		while ($row = $this->db->fetch_array())
		{
			$identifier = new identifier();
			$identifier->id = $row['identifier_ID'];
			$identifier->identifier = $row['identifier_identifier'];
			$identifier->type = $row['identifier_type'];
			$identifier->href = $row['identifier_href'];
			$this->populatecommon($identifier, $row, 'identifier_');
			$identifiers[$row['identifier_identityID']][] = $identifier;
		}
		$this->db->free_result();

		return $identifiers;
	}

	function getnames($ids)
	{
		$idstext = $this->getidstring($ids);

		$this->db->query("
			SELECT * FROM {$this->prefix}name WHERE name_identityID $idstext");

		$names = array();
		while ($row = $this->db->fetch_array())
		{
			$name = new name();
			$name->id = $row['name_ID'];
			$name->surname = $row['name_surname'];
			$name->givennames = $row['name_givennames'];
			$name->title = $row['name_title'];
			$name->surnamefirst = $row['name_surnamefirst'];
			$name->type = $row['name_type'];
			$this->populatecommon($name, $row, 'name_');
			$names[$row['name_identityID']][] = $name;
		}
		$this->db->free_result();

		return $names;
	}

	function getorganisations($ids)
	{
		$idstext = $this->getidstring($ids);

		$this->db->query("
			SELECT * FROM {$this->prefix}organisation WHERE organisation_identityID $idstext");

		$organisations = array();
		while ($row = $this->db->fetch_array())
		{
			$organisation = new organisation();
			$organisation->id = $row['organisation_ID'];
			$organisation->title = $row['organisation_title'];
			$organisation->type = $row['organisation_type'];
			$organisation->reltype = $row['organisation_reltype'];
			$this->populatecommon($organisation, $row, 'organisation_');
			$organisations[$row['organisation_identityID']][] = $organisation;
		}
		$this->db->free_result();

		return $organisations;
	}

	function getpersons($ids, $recursionlimit = 1)
	{	
		$idstext = $this->getidstring($ids);
		$persons = array();
		$fetchidentities = array();

		$this->db->query("
			(SELECT *, 0 AS r FROM {$this->prefix}person WHERE person_identityID $idstext)
			UNION ALL
			(SELECT *, 1 AS r FROM {$this->prefix}person WHERE person_secondidentityID $idstext)
			");

		$done = array();
		while ($row = $this->db->fetch_array())
		{
			$nearid = $row['r'] ? $row['person_secondidentityID'] : $row['person_identityID'];
			$farid  = $row['r'] ? $row['person_identityID'] : $row['person_secondidentityID'];
			$person = new person();
			$person->id = $row['person_ID'];
			$person->type = $row['person_type'];
			$person->person = new identity();
			$person->person->id = $farid;
			$fetchidentities[] = $person->person->id;
			$this->populatecommon($person, $row, 'person_');

				// don't duplicate - should never matter
				// unless an identity got a reference to itself somehow
			if (!empty($done[$nearid][$farid])) continue; 
			$done[$nearid][$farid] = true;

			$persons[$nearid][] = $person;
		}
		$this->db->free_result();

		if (!empty($fetchidentities))
		{
			$identities = $this->getidentities(array_unique($fetchidentities), $recursionlimit - 1);
			foreach ($persons as $firstid => $val)
				foreach ($val as $index => $person)
					if (isset($identities[$person->person->id]))
					{
						foreach ($identities[$person->person->id] as $name=>$pval)
							$persons[$firstid][$index]->person->$name = $pval;
					}	
		}

		return $persons;
	}

	function getresources($ids)
	{
		$idstext = $this->getidstring($ids);

		$this->db->query("
			SELECT * FROM {$this->prefix}resource WHERE resource_identityID $idstext");

		$resources = array();
		while ($row = $this->db->fetch_array())
		{
			$resource = new resource();
			$resource->id = $row['resource_ID'];
			$resource->title = $row['resource_title'];
			$resource->type = $row['resource_type'];
			$resource->reltype = $row['resource_reltype'];
			$this->populatecommon($resource, $row, 'resource_');
			$resources[$row['resource_identityID']][] = $resource;
		}
		$this->db->free_result();

		return $resources;
	}

	function matchidentity($identity)
	// searches for a matching identity to the information given in $identity
	// and returns either a new identity or a merged identity with the one given
	// records uncertainty information for the identity
	{
		$controls = &controls::getinstance();
		$indexlookup = $controls->getcontrol('indexlookup');
		$editidentity = $controls->getcontrol('editidentity');

		$ids = $indexlookup->findmatches($identity);
		$identities = $this->getidentities($ids);

		$results = $identity->findmatch($identities);

		$isnew = false;

		if ($results['foundmatch'])
		{
			// merge
			$newidentity = $identities[$results['foundmatch']];
			$oldidentity = clone($newidentity);
			$newidentity->merge($identity);

			if ($newidentity != $oldidentity // only flag if it's changed
				&& ($results['uncertain'] || $results['othercandidates']))
					$newidentity->status = 'uncertainmatch';

			// todo this will create false positive if things like the level of
			// recursion differ, which may never actually change things in the DB
			// we could create a better way of testing 'sameness' of two identities
		}
		else
		{
			// create new
			$isnew = true;
			$newidentity = $identity;

			$newidentity->status = 'nomatch';
		}

		return array(
			'newidentity' => $newidentity,
			'isnew' => $isnew
			);
	}

	function getcountbystatus($status, $cutoff = null)
		// gets the number of identities with this status.  optionally you may specify
		// a cutoff time (unix timestamp) where only identities that have not been
		// modified since before this date are counted
	{
		$statussl = addslashes($status);
		$andcutoff = $cutoff !== null ? ("AND identity_lastmodifieddate<".(int)$cutoff) : '';

		list($count) = $this->db->query_single("
			SELECT
				COUNT(*) AS c
			FROM
				{$this->prefix}identity
			WHERE
				identity_status='$statussl'
				$andcutoff
			");

		return $count;
	}

	function getidsbystatus($status, $sortdesc = false, $limit = 1000)
	// returns ids of identities matching the status $status, eg. 
	// 'requestedverification'.  sorted by last modified date.  if $sortdesc is true
	// it is sorted in descending order (newest first)
	{
		$statussl = addslashes($status);
		$sortorder = $sortdesc ? "DESC" : '';
		$limit = (int)$limit;

		$this->db->query("
			SELECT
				identity_ID
			FROM
				{$this->prefix}identity
			WHERE
				identity_status='$statussl'
			ORDER BY
				identity_status $sortorder,
				identity_lastmodifieddate $sortorder
			LIMIT $limit
			");

		$ids = array();
		while ($row = $this->db->fetch_array())
			$ids[] = $row['identity_ID'];

		$this->db->free_result();

		return $ids;
	}

	function getidsrecentlymodified($limit = 1000)
	{
		$this->db->query("
			SELECT
				identity_ID
			FROM
				{$this->prefix}identity
			ORDER BY
				identity_lastmodifieddate DESC
			LIMIT $limit
			");

		$ids = array();
		while ($row = $this->db->fetch_array())
			$ids[] = $row['identity_ID'];

		$this->db->free_result();

		return $ids;
	}

	function getidsmodifiedsince($mindate = 0, $limit = 1000, $maxdate = null, $showdeleted = true)
		// returns ids of identities that have been modified since the date given in
		// $mindate (or, if mindate is omitted, all identities) sorted by their
		// last modification date.
	{
		$mindate = (int)$mindate;
		$limit = (int)$limit;

		$ids = array();

		// get live identities
		$andmaxdate = $maxdate ? ("AND identity_lastmodifieddate<" . (int)$maxdate) :
			"";
		$this->db->query("
			SELECT
				identity_ID AS id,
				identity_lastmodifieddate AS date
			FROM
				{$this->prefix}identity FORCE INDEX (identity_lastmodifieddate)
			WHERE
				identity_lastmodifieddate>=$mindate
				$andmaxdate
			LIMIT $limit
			");
		while ($row = $this->db->fetch_array())
			$ids[$row['id']] = $row['date'];
		$this->db->free_result();

		if ($showdeleted)
		{
			// get deleted/merged identities
			$andmaxdate = $maxdate ? ("AND identity_alias_lastmodifieddate<" . (int)$maxdate) :
				"";
			$this->db->query("
				SELECT
					identity_alias_ID AS id,
					identity_alias_lastmodifieddate AS date
				FROM
					{$this->prefix}identity_alias FORCE INDEX (identity_alias_lastmodifieddate)
				WHERE
					identity_alias_lastmodifieddate>=$mindate
					$andmaxdate
				LIMIT $limit
				");
			while ($row = $this->db->fetch_array())
				if (!isset($ids[$row['id']]))
					$ids[$row['id']] = $row['date'];
			$this->db->free_result();
		}

		asort($ids);

		return array_slice(array_keys($ids), 0, $limit);
	}

	private function populatecommon(&$obj, $row, $prefix)
	{
		$obj->startyear = $row[$prefix.'startyear'];
		$obj->endyear = $row[$prefix.'endyear'];
		$obj->comment = $row[$prefix.'comment'];
		$obj->incidence = $row[$prefix.'incidence'];
		$obj->score = $row[$prefix.'score'];
		$obj->verified = $row[$prefix.'verified'];
	}

	private function getidstring($ids)
	{
		if (!is_array($ids)) $ids = array($ids);
		$ids = array_map('intval', $ids);
		$idstext = count($ids) <> 1 ? ('IN (' . implode(',', $ids) . ')') : ('=' . current($ids));
		return $idstext;
	}

	/*
	function getaliasesmodifiedsince($mindate = 0, $limit = 1000, $maxdate = null)
		// returns ids of identities that have been deleted/merged since the date given in
		// $mindate (or, if mindate is omitted, all aliases) sorted by their
		// last modification date.
	{
		$mindate = (int)$mindate;
		$limit = (int)$limit;

		$andmaxdate = $maxdate ? ("AND identity_alias_lastmodifieddate<" . (int)$maxdate) :
			"";

		$this->db->query("
			SELECT
				identity_alias_ID
			FROM
				{$this->prefix}identity_alias FORCE INDEX (identity_alias_lastmodifieddate)
			WHERE
				identity_alias_lastmodifieddate>=$mindate
				$andmaxdate
			LIMIT $limit
			");

		// todo this relies on the rows being returned in sorted order simply by
		// using the index - supplying an ORDER BY forces a filesort.  this isn't
		// likely to be stable always - find a better way of doing it.

		$ids = array();
		while ($row = $this->db->fetch_array())
			$ids[] = $row['identity_ID'];

		$this->db->free_result();

		return $ids;
	}
	 */
}

?>
