<?php

/*
	identity - a class that represents a person's identity
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/name.inc.php');
require_once(RESOURCE_DIR . 'includes/classification.inc.php');
require_once(RESOURCE_DIR . 'includes/event.inc.php');
require_once(RESOURCE_DIR . 'includes/identifier.inc.php');
require_once(RESOURCE_DIR . 'includes/organisation.inc.php');
require_once(RESOURCE_DIR . 'includes/person.inc.php');
require_once(RESOURCE_DIR . 'includes/resource.inc.php');

require_once(RESOURCE_DIR . 'includes/textfuzzy.inc.php');
require_once(RESOURCE_DIR . 'includes/probability.inc.php');

class identity
{
	public $id = null;
	public $birthyear = null;
	public $comment = null;
	public $createddate = null;
	public $lastmodifieddate = TIMENOW;
	public $affiliation = 'none';
	public $status = 'normal';

	public $names = array();
	public $classifications = array();
	public $events = array();
	public $identifiers = array();
	public $organisations = array();
	public $persons = array();
	public $resources = array();

	function getfields($type = 'identity')
	{
		static $affiliation_array = array(
			'affiliated' => 'Affiliated',
			'previouslyaffiliated' => 'Previously affiliated',
			'none' => 'No affiliation',
			);

		if ($type == 'identity') return array(
			array('affiliation', 'select', 'affiliation', 'Is this person affiliated with your institution?', null, true, $affiliation_array),
			array('birthyear', 'int', 'birth year', 'Year this person was born, if known', 4, false, array('min' => 1400, 'max' => date('Y'))),
		);
		if ($type == 'comment') return array(
			array('comment', 'string', 'comments', 'Add one or more optional comments to this record.', 65535, false, array('multiline' => true, 'rows' => 12)),
			);
	}

	function getXML($document = null, $elementname = 'identity')
	// Create a DomElement object representing this identity
	{
		if ($document===null) $document = new DomDocument();
		$el = $document->createElement($elementname);
		
		foreach ($this->names as $entry)
			$el->appendChild($entry->getXML($document));

		foreach ($this->classifications as $entry)
			$el->appendChild($entry->getXML($document));

		foreach ($this->events as $entry)
			$el->appendChild($entry->getXML($document));

		foreach ($this->identifiers as $entry)
			$el->appendChild($entry->getXML($document));

		foreach ($this->organisations as $entry)
			$el->appendChild($entry->getXML($document));

		foreach ($this->persons as $entry)
			$el->appendChild($entry->getXML($document));

		foreach ($this->resources as $entry)
			$el->appendChild($entry->getXML($document));

		if ($this->id) $el->setAttribute('id', $this->id);
		if ($this->comment != '') $el->setAttribute('comment', $this->comment);
		if ($this->birthyear) $el->setAttribute('birthyear', $this->birthyear);
		$el->setAttribute('affiliation', $this->affiliation);
		if ($this->createddate)
		 	$el->setAttribute('createddate', gmdate("Y-m-d\TH:i:s\Z", $this->createddate));
		if ($this->lastmodifieddate) 
			$el->setAttribute('lastmodifieddate', gmdate("Y-m-d\TH:i:s\Z", $this->lastmodifieddate));
		$el->setAttribute('status', $this->status);
		return $el;
	}

	function getOAIDC($document = null)
	{
		if ($document === null) $document = new DomDocument();
		$el = $document->createElement("oai_dc:dc");
		$el->setAttribute('xmlns:oai_dc', 'http://www.openarchives.org/OAI/2.0/oai_dc/');
		$el->setAttribute('xmlns:dc', 'http://purl.org/dc/elements/1.1/');
		$el->setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
		$el->setAttribute('xsi:schemaLocation', 'http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd');

		$el->appendChild($document->createElement("dc:identifier", htmlspecialchars("nicnames:" . $this->id)));

		$this->foreachrelation(array($this, 'getOAIDCfields'), array($document, $el));

		if ($this->birthyear)
			$el->appendChild($document->createElement("dc:description", "Year of birth: " . (int)$this->birthyear));
		$el->appendChild($document->createElement("dc:subject", htmlspecialchars("affiliationstatus:" . $this->affiliation)));

		return $el;
	}

	function getOAIDCfields($obj, $type, $id, $arg)
	// for use in callback
	{
		$document = $arg[0];
		$el = $arg[1];

		if ($type == 'name')
			$el->appendChild($document->createElement("dc:title", htmlspecialchars($obj->getstandardname())));
		elseif ($type == 'identifier' && $obj->type != 'other')
			$el->appendChild($document->createElement("dc:identifier", htmlspecialchars($obj->type . ":" . $obj->identifier)));
		elseif ($type == 'classification' && $obj->code != "" && $obj->type != 'other')
			$el->appendChild($document->createElement("dc:subject", htmlspecialchars($obj->type . ":" . $obj->code)));

		$child = $document->createElement("dc:description", htmlspecialchars($obj->getplaintext()));
		$el->appendChild($child);
	}

	function setfromXML($el)
	// populate this object with properties from the supplied DOM XML element
	{
		if ($el->hasAttribute('id')) $this->id = $el->getAttribute('id');
		if ($el->hasAttribute('birthyear')) $this->birthyear = $el->getAttribute('birthyear');
		if ($el->hasAttribute('comment')) $this->comment = $el->getAttribute('comment');
		if ($el->hasAttribute('affiliation')) $this->affiliation = $el->getAttribute('affiliation');
		if ($el->hasAttribute('createddate')) $this->createddate = strtotime($el->getAttribute('createddate'));
		if ($el->hasAttribute('lastmodifieddate')) $this->lastmodifieddate = strtotime($el->getAttribute('lastmodifieddate'));
		if ($el->hasAttribute('status')) $this->status = $el->getAttribute('status');

		for ($i = 0, $len = $el->childNodes->length; $i < $len; $i++)
		{
			$node = $el->childNodes->item($i);
			$nodename = $node->nodeName;
			switch ($nodename)
			{
			case 'name':
			case 'classification':
			case 'event':
			case 'identifier':
			case 'organisation':
			case 'person':
			case 'resource':
				$obj = new $nodename();
				$obj->setfromXML($node);
				$this->{$nodename.'s'}[] = $obj;
				break;
			}
		}
	}

	function findmatch($identities)
		// given an array of identity objects, find the one that is the best
		// match to this identity or, if none are a good enough match, return
		// nothing
	{
		$max_id = null;
		$max_val = 0;
		$candidates = 0;
		
		foreach ($identities as $id => $identity)
		{
			$score = $this->similarityto($identity);
			if ($score > 0.6) $candidates += 1;
			if ($score > $max_val)
			{
				$max_id = $id;
				$max_val = $score;
			}
		}

		return array(
			'foundmatch' => $candidates ? $max_id : null,
			'bestscore' => $max_val,
			'uncertain' => $max_val > 0.1 && $max_val <= 0.9,
			'othercandidates' => $candidates > 1,
			);
	}

	function similarityto($other)
		// compares this identity to some other identity and returns a score
		// between 0 and 1 (inclusive) indicating how similar they are.  more
		// similar matches will result in higher scores.  The system tries to
		// take an intelligent and fuzzy approach to determining what is a
		// good match.
	{
		$factors = array();

		if ($this->id && $other->id)
		{
			if ($this->id == $other->id)
				$factors['id'] = array(1, 400);
			else
				$factors['id'] = array(0, 80);
		}
		if ($this->birthyear && $other->birthyear)
			$factors['birthyear'] = array($this->birthyear == $other->birthyear, 8);
		if ($this->comment != '' && $other->comment != '')
		{
			$comm1 = new textfuzzy($this->comment);
			$comm2 = new textfuzzy($this->comment);
			$factors['comment'] = array($comm1->similarityto($comm2), 2);
		}

		if ($this->affiliation && $other->affiliation)
			$factors['affiliation'] = array($this->affiliation == $other->affiliation, 2);

		$factors['name']           = $this->compareall($this->names, $other->names, 200);
		$factors['classification'] = $this->compareall($this->classifications, $other->classifications, 18);
		$factors['event']          = $this->compareall($this->events, $other->events, 50);
		$factors['identifier']     = $this->compareall($this->identifiers, $other->identifiers, 200);
		$factors['organisation']   = $this->compareall($this->organisations, $other->organisations, 20);
		$factors['person']         = $this->compareall($this->persons, $other->persons, 15);
		$factors['resource']       = $this->compareall($this->resources, $other->resources, 60);

		$p = probability::combinedcorrected($factors);
		return $p;
	}

	private function compareall($firstcol, $secondcol, $weightscore = 100)
		// compares each 'relation' in the first array with each 'relation'
		// in the second array.  the relations can be of any class, but all
		// need to be the same class each time
		// returns array (0 => score, 1 => weight)
	{
		$scores = array();

		$rows = count($firstcol);
		$cols = count($secondcol);

		if (!$rows || !$cols) return array(0, $weightscore * 0.02);

		foreach ($firstcol as $rel1)
			foreach ($secondcol as $rel2)
				$scores[] = $rel1->similarityto($rel2);

		asort($scores);

		// pair off the matches (don't match one from either side more than once)
		$factors = array();
		$cumweight = 150;
		while (!empty($scores))
		{
			$score = end($scores);
			$key = min(array_keys($scores, $score));
			$factors[] = array($score, $cumweight);
			$row = floor($key / $cols);
			$col = $key - ($row * $cols);
			for ($i = 0; $i < $rows; $i++) 
				if (isset($scores[$i * $cols + $col])) unset($scores[$i * $cols + $col]);
			for ($i = 0; $i < $cols; $i++)
				if (isset($scores[$row * $cols + $i])) unset($scores[$row * $cols + $i]);

			$cumweight = $cumweight * 0.5;
		}

		$score = probability::combinedcorrected($factors);

		return array($score, $weightscore * ($score > 0.5 ? 1 : 0.5));
	}

	function gettextfieldgroups()
		// returns array of array of text fields suitable for indexing in a search engine
		// each entry is a array($text, $score, $category) where score is a relative weight
		// of the text in being characteristic to this particular resource
	{
		$groups = array(
			'identity' => array(
				array($this->birthyear, 12),
				array($this->comment, 8),
			),
			'name' => array(),
			'event' => array(),
			'organisation' => array(),
			'person' => array(),
			'identifier' => array(),
			'classification' => array(),
			'resource' => array(),
			);

		if ($this->affiliation != 'none') $groups['identity'][] = array($this->id, 100);

		foreach ($this->names as $name) $groups['name'] = array_merge($groups['name'], $name->gettextfields());
		foreach ($this->events as $event) $groups['event'] = array_merge($groups['event'], $event->gettextfields());
		foreach ($this->organisations as $organisation) $groups['organisation'] = array_merge($groups['organisation'], $organisation->gettextfields());
		foreach ($this->persons as $person) $groups['person'] = array_merge($groups['person'], $person->gettextfields());
		foreach ($this->identifiers as $identifier) $groups['identifier'] = array_merge($groups['identifier'], $identifier->gettextfields());
		foreach ($this->classifications as $classification) $groups['classification'] = array_merge($groups['classification'], $classification->gettextfields());
		foreach ($this->resources as $resource) $groups['resource'] = array_merge($groups['resource'], $resource->gettextfields());

		return $groups;
	}
	
	function merge($other)
		// merges another identity into this one.  facts will be combined
		// fairly intelligently, ie facts which are duplicates or very near
		// duplicates will not be duplicated.  For true conflicts, like
		// having a different birthyear, it will keep the local value
	{
		foreach ($other as $name => $value)
		{
			if (
				(!isset($this->$name))
				|| ($name === 'createddate' && $value !== null && $value < $this->createddate)
				|| ($name === 'affiliation' && ($value == 'affiliated' || ($value == 'previouslyaffiliated' && $this->affiliation == 'none')))
				|| (empty($this->$name) && !empty($value))
			)
				$this->$name = $value;
			elseif (
				($name === 'comment'
				&& trim($value) != '' && strpos($this->comment, $value) === false)
			)
				$this->comment .= "\n-\n" . $value;
			elseif ( // check if we have some relations to compare
				(!empty($this->$name) && !empty($value) &&
					in_array($name, array('names', 'events', 'organisations', 'persons', 'identifiers', 'classifications', 'resources')))
			)
			{
				foreach ($value as $farfact)
				{
					$found = null;
					foreach ($this->$name as $id => $nearfact)
					{
						$score = $nearfact->similarityto($farfact);
						if ($score > 0.95)	
						{
							$found = $id;
							break;
						}
					}
					if ($found !== null)
						$this->{$name}[$id]->merge($farfact);
					else
						$this->{$name}[] = $farfact;
				}
			}
		}
	}

	function foreachrelation($callback, $arg = null)
	{
		foreach ($this->names as $id => $obj)           call_user_func($callback, $obj, 'name', $id, $arg); 
		foreach ($this->classifications as $id => $obj) call_user_func($callback, $obj, 'classification', $id, $arg); 
		foreach ($this->organisations as $id => $obj)   call_user_func($callback, $obj, 'organisation', $id, $arg); 
		foreach ($this->identifiers as $id => $obj)     call_user_func($callback, $obj, 'identifier', $id, $arg); 
		foreach ($this->persons as $id => $obj)         call_user_func($callback, $obj, 'person', $id, $arg); 
		foreach ($this->resources as $id => $obj)       call_user_func($callback, $obj, 'resource', $id, $arg); 
		foreach ($this->events as $id => $obj)          call_user_func($callback, $obj, 'event', $id, $arg); 
		return $this;
	}

	function getrelation($type, $id)
	// returns the relation with this type and ID, or NULL if not found
	{
		if (!isset($this->{$type.'s'})) trigger_error('Invalid relation type', E_USER_ERROR);
		foreach ($this->{$type.'s'} as $i => $obj)
			if ($obj->id == $id)
				return $this->{$type.'s'}[$i];

		return null;
	}

	function deleterelation($type, $id)
	// attempts to delete the relation with this type and ID.  Returns true on
	// success or false on failure
	{
		if (!isset($this->{$type.'s'})) trigger_error('Invalid relation type', E_USER_ERROR);
		foreach ($this->{$type.'s'} as $i => $obj)
			if ($obj->id == $id)
		{
			unset($this->{$type.'s'}[$i]);
			return true;
		}
		return false;
	}
	
	function replacerelation($type, $id, $new)
	// replaces the given relation with new.  This could be done directly, but with this
	// method it ensures the new relation will have the same ID.
	// returns true on success or false on failure
	{
		$rel = $this->getrelation($type, $id);
		$rel = $new;
		$rel->id = $id;
	}

	function anonymise($obj = null, $type = null)
	// this should be done to all identities sourced from a third party, where that
	// third party does not use the same unique identifiers for all identities and relations
	// as this installation.  It removes it, ready for merging or inserting.
	{
		if (!$obj)
		{
			$this->foreachrelation(array($this, 'anonymise'));
			$this->id = null;
			return;
		}

		if ($type == 'person') $obj->person->anonymise();

		$obj->id = null;
	}

	function localise($obj = null, $type = null)
	// filters all relations so that any types or reltypes not valid in the local
	// institution can be filtered (eg. to 'other').
	{
		if (!$obj)
			return $this->foreachrelation(array($this, 'localise'));

		if ($type == 'person') $obj->person->localise();

		if (isset($obj->type) && isset(nicnames_config::$resourcetypes["{$type}_type"])
			&& !isset(nicnames_config::$resourcetypes["{$type}_type"][$obj->type]))
		{
			if (strpos($obj->comment, $obj->type) === false)
			{
				if ($obj->comment != '') $obj->comment .= " ";
				$obj->comment .= "[" . $obj->type . "]";
			}
			$obj->type = isset(nicnames_config::$resourcetypes["{$type}_type"]['other'])
				? 'other' : key(nicnames_config::$resourcetypes["{$type}_type"]);
		}

		if (isset($obj->reltype) && isset(nicnames_config::$resourcetypes["{$type}_reltype"])
			&& !isset(nicnames_config::$resourcetypes["{$type}_reltype"][$obj->reltype]))
		{
			if ($obj->comment != '') $obj->comment .= "; ";
			$obj->comment .= "Unsupported local relation type '" . $obj->reltype . "'";
			$obj->reltype = isset(nicnames_config::$resourcetypes["{$type}_reltype"]['other'])
				? 'other' : key(nicnames_config::$resourcetypes["{$type}_reltype"]);
		}
	}

	function markverified($obj = null, $type = null, $id = null)
	// sets this identity, and all its relations, to verified
	{
		if (!$obj)
		{
			$this->status = 'normal';
			return $this->foreachrelation(array($this, 'markverified'));
		}

		$obj->verified = true;
	}

	function validate()
	// checks validity of identity.  returns empty array on success, or array
	// of errors of form (errorcode => errormsg) on any errors
	// this isn't a comprehensive check of required vals, type checking or anything
	// - it just checks some big picture logic, like the fact an identity must have
	// at least one name
	{
		$errors = array();
		if (count($this->names) < 1)
			$errors['nonames'] = 'An identity must have at least one name.';

		// todo implement other mandatories 
		
		return $errors;
	}

	function sameas($other)
		// should return true if this and the other identity are 'the same' enough such
		// that it shoudn't need updating in database or changing 'last modified' time.
	{
		$checksimple = array(
			'birthyear', 'comment', 'createddate', 'affiliation', 'status');
		foreach ($checksimple as $name) if ($this->$name != $other->$name) return false;

		$cats = array(
			'names', 'classifications', 'events', 'identifiers', 'organisations',
			'persons', 'resources');
		foreach ($cats as $cat)
		{
			$here = $this->$cat;
			$there = $other->$cat;
			if (count($here) != count($there)) return false;
			foreach ($here as $id => $obj)
				if (!isset($there[$id]) || $there[$id] != $here[$id]) return false;
		}

		return true;
	}

	function stripprivate($obj = null, $type = null, $id = null)
	// strips private information - information that the public should not be able to access -
	// about this identity
	{
		if (!$obj)
			return $this->foreachrelation(array($this, 'stripprivate'));

		$varname = $type . 's';
		if (isset(nicnames_config::$privateinformation["{$type}_type"])
			&& in_array($obj->type, nicnames_config::$privateinformation["{$type}_type"]))
		{
			/*if ($type == 'name')
			{
				$obj->surname = '(' . nicnames_config::$resourcetypes['name_type'][$obj->type]
					. ' withheld)';
				$obj->givennames = $obj->title = null;
			}
			else*/
				unset($this->{$varname}[$id]);
		}
	}

	function getrendered($limit = 4, $searchstring = '' /* todo searchstring matching not yet implemented */)
	{
		$this->facts = array();
		$this->scores = array();
		$this->types = array();
		$this->foreachrelation(array($this, 'renderfact'));
		arsort($this->scores);

		$output = array();
		// get highest score name
		foreach ($this->scores as $id => $score) if ($this->types[$id] == 'name')
		{
			$output[] = $this->facts[$id];
			unset($this->scores[$id]);
			$limit--;
			break;
		}

		// if no names then put placeholder
		if (empty($output)) $output[] = array(
			'pre-qualifier' => 'Name',
			'title' => '[no name available]',
			'verified' => true,
			'type' => 'factnotice'
			);

		foreach ($this->scores as $id => $score)
		{
			if ($limit == 0) break;
			$output[] = $this->facts[$id];
			$limit--;
		}

		unset($this->facts);
		unset($this->scores);
		unset($this->types);

		return $output;
	}

	private function renderfact($obj, $type)
	{
		static $catscores = array( // weight likelihood of each turning up
			'name' => 10,
			'classification' => 6,
			'organisation' => 5,
			'identifier' => 3.5,
			'person' => 3,
			'resource' => 2,
			'event' => 1.5,
			);
		$incidence = !empty($obj->incidence) ? $obj->incidence : 1;
		$score = $catscores[$type] * (log($incidence)+1);

		$strings = $obj->getstrings();

		$this->facts[] = $strings;
		$this->scores[] = $score;
		$this->types[] = $type;
	}
}

?>
