<?php

/*
	person - a single person related to an identity
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/relation.inc.php');
require_once(RESOURCE_DIR . 'includes/probability.inc.php');

class person extends relation
{
	public $id = null;
	public $type;
	public $person;  // type 'identity'
	 
	function getfields()
	{
		return array_merge(array(
			array('type', 'select', 'type', 'How is the person related to the current identity?', null, true, nicnames_config::$resourcetypes['person_type']),
			array('person', 'mixed', 'person', 'Specify the details of this person', null, true, array('identitytype' => true)),
			), $this->getcommonfields());
	}

	function getXML($document = null)
	// Create a DomElement object representing this person
	{
		if ($document===null) $document = new DomDocument();
		$el = $document->createElement('person');

		$ident = $this->person->getXML($document);
		$el->appendChild($ident);

		$el->setAttribute('type', $this->type);
		if ($this->id) $el->setAttribute('id', $this->id);
		$el = $this->getcommonpropertiesXML($el);
		return $el;
	}

	function setfromXML($el)
	// populate this object with properties from the supplied DOM XML element
	{
		if ($el->hasAttribute('id')) $this->id = $el->getAttribute('id');
		
		$this->type = $el->getAttribute('type');

		$this->person = new identity();

		for ($i = 0, $len = $el->childNodes->length; $i < $len; $i++)
		{
			$node = $el->childNodes->item($i);

			if ($node->nodeName == 'identity')
				$this->person->setfromXML($node);
		}
		
		$this->setcommonfromXML($el);
	}
	
	function similarityto($other)
	{
		$factors = array();

		$factors['person'] = array($this->person->similarityto($other->person), 600);
		$factors['type'] = array($this->type == $other->type, $factors['person'][0] > 0.5 ? 80 : 4);

		$factors['common'] = $this->commonsimilarityfactor($other);

		$score = probability::combinedcorrected($factors);
		return $score;
	}

	function getstrings()
		// returns array of strings suitable for human-readable rendering of this
		// piece of informtion.  Contains such fields as 'title', 'subtitle', 
		// 'pre-qualifier', 'post-qualifier', 'comment', etc
	{
		$type = isset(nicnames_config::$resourcetypes['person_type'][$this->type]) ?
			nicnames_config::$resourcetypes['person_type'][$this->type] : $this->type;
		list($strings) = $this->person->getrendered(1);
		$postqual = (isset($strings['pre-qualifier']) && $strings['pre-qualifier'] != '')
			? ($strings['pre-qualifier'] . 
				(isset($strings['post-qualifier']) && $strings['post-qualifier'] != '' ? (', ' . $strings['post-qualifier']) : ''))
			: $strings['post-qualifier'];

		return array(
			'pre-qualifier' => $type,
			'post-qualifier' => $postqual,
			'identityid' => $this->person->id ? $this->person->id : null,
		) + $this->getcommonstrings() + $strings;
	}
}

?>
