<?php

/*
	resource - a single resource related to an identity
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/relation.inc.php');

class resource extends relation
{
	public $id = null;
	public $title = null;
	public $type;
	public $reltype;
	 
	function getfields()
	{
		return array_merge(array(
			array('type', 'select', 'type', 'Choose the type of resource', null, true, nicnames_config::$resourcetypes['resource_type']),
			array('title', 'string', 'resource title', 'Please specify the title of this resource', 255, true, array('size' => 72)),
			array('reltype', 'select', 'contribution', 'How is this person related to this resource?', null, true, nicnames_config::$resourcetypes['resource_reltype']),
			), $this->getcommonfields());
	}

	function getXML($document = null)
	// Create a DomElement object representing this resource
	{
		if ($document===null) $document = new DomDocument();
		$el = $document->createElement('resource', $this->title != '' ? htmlspecialchars($this->title) : '');
		if ($this->id) $el->setAttribute('id', $this->id);
		$el->setAttribute('type', $this->type);
		$el->setAttribute('reltype', $this->reltype);
		$el = $this->getcommonpropertiesXML($el);
		return $el;
	}

	function setfromXML($el)
	// populate this object with properties from the supplied DOM XML element
	{
		if ($el->hasAttribute('id')) $this->id = $el->getAttribute('id');
		if ($el->nodeValue != '') $this->title = $el->nodeValue;
		$this->type = $el->getAttribute('type');
		$this->reltype = $el->getAttribute('reltype');

		$this->setcommonfromXML($el);
	}

	function similarityto($other)
	{
		$factors = array();

		if ($this->title != '' && $other->title != '')
		{
			$title1 = new textfuzzy($this->title);
			$title2 = new textfuzzy($other->title);
			$factors['title'] = array($title1->similarityto($title2), 100);
		}
		elseif ($this->title != '' || $other->title != '')
			$factors['title'] = array(0, 3);
		else
			$factors['title'] = array(0, 1.5);

		$factors['type'] = array($this->type == $other->type, 4);
		$factors['reltype'] = array($this->reltype == $other->reltype, 2);

		$factors['common'] = $this->commonsimilarityfactor($other);
	
		return probability::combinedcorrected($factors);
	}
	
	function getstrings()
		// returns array of strings suitable for human-readable rendering of this
		// piece of informtion.  contains such fields as 'title', 'subtitle', 
		// 'pre-qualifier', 'post-qualifier', 'comment', etc
	{
		$type = isset(nicnames_config::$resourcetypes['resource_type'][$this->type]) ?
			nicnames_config::$resourcetypes['resource_type'][$this->type] : $this->type;
		$reltype = isset(nicnames_config::$resourcetypes['resource_reltype'][$this->reltype]) ?
			nicnames_config::$resourcetypes['resource_reltype'][$this->reltype] : $this->reltype;
		return array(
			'title' => $this->title == '' ? null : $this->title,
			'pre-qualifier' => $type,
			'post-qualifier' => $reltype,
		) + $this->getcommonstrings();
	}
}

?>
