<?php

/*
	action_edit - for viewing/editing an identity and all its relations
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

class action_edit extends module
{
	function display()
	{
		$this->template->vars['title'] = $this->fullaccess ? 'Edit identity' : 'View identity';

		$identityid = $this->context->load_var('identityid', 'REQUEST', 'int');
		if (!$identityid || $identityid <= 0)
		{
			$this->status = array(400, 'Bad Request');
			$this->template->vars['notices'][] = "Error: no identity was specified.  The link you followed appears to be invalid - please go back and try again.";
			return null;
		}

		// messages
		$m = $this->context->load_var('m', 'REQUEST', 'name');
		if ($m == 'createsuccess') $this->template->vars['notices'][] = 'The new identity was successfully created.  You may now edit it further below.';
		elseif ($m == 'deleterelsuccess') $this->template->vars['notices'][] = 'The piece of information you chose has been removed from this identity.';
		elseif ($m == 'updaterelsuccess') $this->template->vars['notices'][] = 'Your changes to this identity have been saved.';
		elseif ($m == 'markverifiedsuccess') $this->template->vars['notices'][] = 'The identity was successfully marked as verified.';
		elseif ($m == 'updateidentitysuccess') $this->template->vars['notices'][] = 'Your changes to this identity have been saved.';
		elseif ($m == 'previouslyaffiliatedsuccess') $this->template->vars['notices'][] = 'This identity has been marked as previously affiliated with this institution.';
		elseif ($m == 'noaffiliationsuccess') $this->template->vars['notices'][] = 'This identity has been marked as never having been affiliated with this institution.';
		elseif ($m == 'mergesuccess') $this->template->vars['notices'][] = 'The two identities have been successfully merged.';
		if ($m) $this->vars['message'] = $m;

		$fetchidentities = $this->controls->getcontrol('fetchidentities');
		$identities = $fetchidentities->getidentities(array($identityid), 1);
		if (empty($identities) || count($identities) != 1)
		{
			$this->vars['error'] = 'nomatches';
			$this->status = array(404, 'Not Found');
			$this->template->vars['notices'][] = "Error: identity not found.  The link you followed appears to point to an identity which no longer exists - please go back and try another identity.";
			return null;
		}
		$identity = reset($identities);
		if (!$this->fullaccess) $identity->stripprivate();
		$this->identities = array($identity);

		// set common template vars
		$this->template->vars['action_identityid'] = $identity->id;
		$this->template->vars['action_identity'] = $identity;
		$this->template->vars['action_categories'] = array(
			'name' => 'name',
			'classification' => 'classification',
			'organisation' => 'organisation',
			'identifier' => 'identifier',
			'person' => 'related person',
			'resource' => 'resource',
			'event' => 'event',
		);
		$affiliation = array(
			'affiliated' => 'affiliated',
			'previouslyaffiliated' => 'previously affiliated',
			'none' => 'not affiliated',
		);

		if (!($identity instanceof identity))
			return $this->showdeletedidentity($identity);


		$this->template->vars['action_affiliationstatus'] = $affiliation[$identity->affiliation];

		$method = $this->restmethod();
		if ($method == 'PUT') return $this->replaceidentity($identity);
		elseif ($method == 'DELETE') return $this->deleteidentitynow($identity);

		// type of edit screen depends on 'type' var
		$type = $this->context->load_var('type', 'REQUEST', 'name');

		if (!$type) return $this->showidentity($identity);
		elseif ($type == 'identity') return $this->editidentity($identity);
		elseif ($type == 'comment') return $this->editidentity($identity, 'comment');
		elseif ($type == 'merge') return $this->mergeidentity($identity);
		else return $this->editrelation($identity, $type);
	}

	function showdeletedidentity($identity)
	{
		if ($this->format == 'html')
			return $this->template->runtemplate('action_edit_showdeleted');
	}

	function showidentity($identity)
	{
		if ($identity instanceof identity &&
			$this->context->load_var('result_verify', 'POST', 'yesno'))
			$this->markverified($identity);

		if (!$this->fullaccess) $this->maxage = 3600;
		if ($this->format == 'html')
			return $this->template->runtemplate('action_edit');
	}

	function markverified($identity)
	{
		if (!$this->fullaccess) return $this->denyaccess('mark this identity as verified');
		$identity->markverified();
		if (count($errors = $identity->validate()))
		{
			$this->template->vars['notices'][] = 'There was a problem saving the identity.';
			foreach ($errors as $msg) $this->template->vars['notices'][] = $msg;
		}
		else
		{
			$editidentity = $this->controls->getcontrol('editidentity');
			$result = $editidentity->updateidentity($identity);
			if (!$result)
			{
				$this->status = array(500, 'Internal Server Error');
				$this->vars['error'] = 'notupdated';
				$this->template->vars['notices'][] = 'Error: the identity could not be updated in the NicNames database due to an unknown error.';
			}
			else
				$this->context->redirect(URL_PATH . "?a=edit&identityid={$identity->id}&f={$this->format}&m=markverifiedsuccess");
		}
	}

	function deleteidentitynow($identity)
	{
		if (!$this->fullaccess) return $this->denyaccess('delete this identity');

		$editidentity = $this->controls->getcontrol('editidentity');
		$result = $editidentity->deleteidentity($identity);
		if (!$result)
		{
			$this->status = array(500, 'Internal Server Error');
			$this->template->vars['notices'][] = 'Error: the identity could not be deleted in the NicNames database due to an unknown error.';
		}
		else
			$this->context->redirect(URL_PATH . "?f={$this->format}&m=deleteidentitysuccess");
	}

	function replaceidentity($identity)
	{
		if (!$this->fullaccess) return $this->denyaccess('replace this identity');

		$identities = $this->getxmlidentities('php://input');
		if (count($identities) != 1)
		{
			$this->status = array(400, 'Bad Request');
			$this->vars['error'] = 'wrongidentitycount';
			$this->templates->vars['notices'][] = 'This action accepts one and only one identity only.';
			return;
		}
		$newidentity = reset($identities);
		if (!$newidentity->id) $newidentity->id = $identity->id;
		if ($newidentity->id != $identity->id)
		{
			$this->status = array(400, 'Bad Request');
			$this->vars['error'] = 'wrongidentity';
			$this->template->vars['notices'][] = 'The submitted identity had a different ID to the one being edited.';
			return;
		}

		$editidentity = $this->controls->getcontrol('editidentity');
		$result = $editidentity->updateidentity($newidentity);
		if (!$result)
		{
			$this->status = array(500, 'Internal Server Error');
			$this->template->vars['notices'][] = 'Error: the identity could not be updated in the NicNames database due to an unknown error.';
		}
		else
			$this->context->redirect(URL_PATH . "?a=edit&identityid={$identity->id}&f={$this->format}&m=updateidentitysuccess");
	}

	function editidentity($identity, $type = 'identity')
	{
		if (!$this->fullaccess) return $this->denyaccess('edit this identity');
		$delete = $this->context->load_var('delete', 'REQUEST', 'yesno');
		if ($delete) return $this->confirmdeleteidentity($identity);

		$this->vars['fields'] = $fields = $identity->getfields($type);
		$this->vars['errors'] = $errors = $this->validateobject($identity, $fields);
		
		if ($this->context->load_var('result_submit', 'POST', 'yesno'))
		{
			$this->template->vars['action_showerrors'] = true;
			if (!empty($errors))
			{
				$this->vars['error'] = 'validationerror';
				$this->template->vars['notices'][] = 'There were one or more problems with the information you gave.  Please correct the below errors and try again.';
			}
			elseif (count($errors = $identity->validate()))
			{
				$this->template->vars['notices'][] = 'There was a problem saving the identity.';
				foreach ($errors as $msg) $this->template->vars['notices'][] = $msg;
			}
			else
			{
				$editidentity = $this->controls->getcontrol('editidentity');
				$result = $editidentity->updateidentity($identity);
				if (!$result)
				{
					$this->status = array(500, 'Internal Server Error');
					$this->template->vars['notices'][] = 'Error: the identity could not be updated in the NicNames database due to an unknown error.';
				}
				else
					$this->context->redirect(URL_PATH . "?a=edit&identityid={$identity->id}&f={$this->format}&m=updateidentitysuccess");
			}
		}

		$this->template->vars['action_type'] = $type;
		$this->template->vars['action_fields'] = $fields;
		$this->template->vars['action_errors'] = $errors;

		return $this->template->runtemplate('action_edit_editidentity');
	}

	function mergeidentity($identity)
	{
		if (!$this->fullaccess) return $this->denyaccess('merge this identity with another');
		if ($this->context->load_var('result_merge', 'POST', 'yesno'))
		{
			$mergeidentityid = $this->context->load_var('selectedidentity', 'REQUEST', 'int');

			if ($mergeidentityid == $identity->id)
			{
				$this->vars['error'] = 'mergeselectionerror';
				$this->template->vars['notices'][] = 'The identity you selected to merge with is the same identity you will be merging.  Please choose another identity.';
			}
			elseif (!$mergeidentityid)
			{
				$this->vars['error'] = 'mergeselectionerror';
				$this->template->vars['notices'][] = 'Please search for, and then select, an identity to merge with, and try again.';
			}
			else
			{
				$fetchidentities = $this->controls->getcontrol('fetchidentities');
				$identities = $fetchidentities->getidentities(array($mergeidentityid), 0);
				if (empty($identities))
				{
					$this->vars['error'] = 'mergeselectionerror';
					$this->template->vars['notices'][] = 'The identity you selected for merging could not be retrieved.  Please try again.';
				}
				else
				{
					$editidentity = $this->controls->getcontrol('editidentity');
					$newidentity = current($identities);
					$newidentity->merge($identity);
					$result = $editidentity->updateidentity($newidentity);
					if (!$result)
					{
						$this->status = array(500, 'Internal Server Error');
						$this->template->vars['notices'][] = 'The identity could not be merged for some unknown reason.  Please try again.';
					}
					else
					{
						$result = $editidentity->deleteidentity($identity, 'merged', $newidentity->id);
						if (!$result)
						{
							$this->status = array(500, 'Internal Server Error');
							$this->template->vars['notices'][] = 'An error occurred when trying to remove the existing identity.  You may have to try merging again.';
						}
						else
							$this->context->redirect(URL_PATH . "?a=edit&identityid={$newidentity->id}&f={$this->format}&m=mergesuccess");
					}
				}
			}
		}

		$this->findidentities(6, false, 8);

		return $this->template->runtemplate('action_edit_merge');
	}

	function confirmdeleteidentity($identity)
	{
		if (!$this->fullaccess) return $this->denyaccess('delete this identity');
		$deleteaction = $this->context->load_var('deleteaction', 'REQUEST', 'name');

		if ($this->context->load_var('result_delete', 'POST', 'yesno'))
		{
			switch ($deleteaction)
			{
				case 'merge':
					return $this->context->redirect(URL_PATH . "?a=edit&identityid={$identity->id}&type=merge&f={$this->format}");
				case 'permanent':
					$this->deleteidentitynow($identity);
					break;
				case 'previouslyaffiliated':
				case 'noaffiliation':
					$identity->affiliation = $deleteaction == 'noaffiliation' ? 'none' : 'previouslyaffiliated';
					if (count($errors = $identity->validate()))
					{
						$this->vars['error'] = 'consistencyerror';
						$this->vars['errors'] = $errors;
						$this->template->vars['notices'][] =
							'There was a problem saving the identity.';
						foreach ($errors as $msg) $this->template->vars['notices'][] = $msg;
					}
					else
					{
						$editidentity = $this->controls->getcontrol('editidentity');
						$result = $editidentity->updateidentity($identity);
						if (!$result)
						{
							$this->status = array(500, 'Internal Server Error');
							$this->template->vars['notices'][] = 'Error: the identity could not be updated in the NicNames database due to an unknown error.';
						}
						else
							$this->context->redirect(URL_PATH . "?a=edit&identityid={$identity->id}&f={$this->format}&m={$deleteaction}success");
					}
					break;
				default:
					$this->template->vars['notices'][] = 'Please select one of the below types of deletion.';
			}
		}

		return $this->template->runtemplate('action_edit_deleteidentity');
	}

	function editrelation($identity, $type)
	{
		if (!$this->fullaccess) return $this->denyaccess('edit this identity information');
		$delete = $this->context->load_var('delete', 'REQUEST', 'yesno');
		$relid = $this->context->load_var('relid', 'REQUEST', 'int');
		$new = $this->context->load_var('new', 'REQUEST', 'yesno');

		if (!$new)
		{
			$relation = $identity->getrelation($type, $relid);
			if (!$relation)
			{
				$this->status = array(404, 'Not Found');
				$this->vars['error'] = 'notfound';
				$this->template->vars['notices'][] = 'The piece of information you are attempting to edit no longer exists.  You may have followed an invalid link, or someone may have edited it moments ago.';
				return;
			}
		}
		else
		{
			$relation = new $type();
			$relation->verified = true;
			$identity->{$type.'s'}[] = $relation;
		}
		$this->vars['fields'] = $fields = $relation->getfields();
		$this->vars['errors'] = $errors = $this->validateobject($relation, $fields);
		
		$chooseperson = $this->context->load_var('result_chooseperson', 'POST', 'mixed');
		if (!empty($chooseperson))
		{
			$fieldname = key($chooseperson);
			foreach ($fields as $id => $field) if ($field[0] == $fieldname)
			{
				$required = $field[5];
				break;
			}

			if (!$this->context->load_var('result_chosen', 'POST', 'yesno'))
			{
				return $this->chooseperson("$fieldname", $required, "?a=edit&identityid={$identity->id}&type=$type&relid=$relid&new=$new", "related person");
			}
		}

		if ($delete) return $this->confirmdeleterelation($identity, $type, $relation);

		if ($this->context->load_var('result_submit', 'POST', 'yesno'))
		{
			$this->template->vars['action_showerrors'] = true;
			if (!empty($errors))
			{
				$this->vars['error'] = 'validationerror';
				$this->template->vars['notices'][] = 'There were one or more problems with the information you gave.  Please correct the below errors and try again.';
			}
			elseif (count($errors = $identity->validate()))
			{
				$this->vars['error'] = 'consistencyerror';
				$this->vars['errors'] = $errors;
				$this->template->vars['notices'][] = 'There was a problem saving the identity.';
				foreach ($errors as $msg) $this->template->vars['notices'][] = $msg;
			}
			else
			{
				$editidentity = $this->controls->getcontrol('editidentity');
				$result = $editidentity->updateidentity($identity);
				if (!$result)
				{
					$this->status = array(500, 'Internal Server Error');
					$this->template->vars['notices'][] = 'Error: the identity could not be updated in the NicNames database due to an unknown error.';
				}
				else
					$this->context->redirect(URL_PATH . "?a=edit&identityid={$identity->id}&f={$this->format}&m=updaterelsuccess");
			}
		}

		$this->template->vars['action_relation'] = $relation;
		$this->template->vars['action_new'] = $new;
		$this->template->vars['action_type'] = $type;
		$this->template->vars['action_fields'] = $fields;
		$this->template->vars['action_errors'] = $errors;

		return $this->template->runtemplate('action_edit_editrelation');
	}

	function confirmdeleterelation($identity, $type, $relation)
	{
		if (!$this->fullaccess) return $this->denyaccess('delete this identity information');
		if ($this->context->load_var('result_delete', 'POST', 'yesno'))
		{
			if (empty($relation->id) || !$identity->deleterelation($type, $relation->id))
			{
				$this->status = array(500, 'Internal Server Error');
				$this->template->vars['notices'][] = 'Error: the piece of information could not be deleted from the identity due to an unknown error.  Please go back to the identity and try again.';
			}
			elseif (count($errors = $identity->validate()))
			{
				$this->vars['error'] = 'consistencyerror';
				$this->vars['errors'] = $errors;
				$this->template->vars['notices'][] = 'There was a problem saving the identity.';
				foreach ($errors as $msg) $this->template->vars['notices'][] = $msg;
			}
			else
			{
				$editidentity = $this->controls->getcontrol('editidentity');
				$result = $editidentity->updateidentity($identity);
				if (!$result)
				{
					$this->status = array(500, 'Internal Server Error');
					$this->template->vars['notices'][] = 'Error: the identity could not be updated in the NicNames database due to an unknown error.';
				}
				else
					$this->context->redirect(URL_PATH . "?a=edit&identityid={$identity->id}&f={$this->format}&m=deleterelsuccess");
			}
		}

		$this->template->vars['action_relation'] = $relation;
		$this->template->vars['action_type'] = $type;

		return $this->template->runtemplate('action_edit_deleterel');
	}
}

?>
