<?php

/*
	action_import - module for manually importing from various sources
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

class action_import extends module
{
	function display()
	{
		$this->template->vars['title'] = 'Import';
		if (!$this->fullaccess) return $this->denyaccess('import identities');

		$this->getimportplugins();

		// when choosing a type, the type is stored in a cookie for later
		// use.  this is a session cookie only.
		$type = $this->context->load_var('type', 'REQUEST', 'name');
		if ($type === null)
			$type = $this->context->load_var('action_import_type', 'COOKIE', 'string');
		else
			$this->context->setcookie('action_import_type', $type);
		if ($type === null) $type = 'nicnamesxml';

		$this->template->vars['action_type'] = $vars['action_type'] = $type;
		
		$method = $this->restmethod();
		if ($method == 'POST') return $this->doimport($type, true);

		if ($this->context->load_var('continue', 'REQUEST', 'yesno'))
			return $this->docontinue($type);

		if ($this->context->load_var('doimport', 'REQUEST', 'yesno'))
			return $this->doimport($type);

		return $this->showform();
	}

	function docontinue($type)
	{
		if (!$this->fullaccess) return $this->denyaccess('import identities');
		if (!isset($this->plugins[$type]))
		{
			$this->status = array(404, 'Not Found');
			$this->template->vars['notices'][] = "Error: the import source you chose (importsource_$type) is not currently installed.";
			return $this->showform();
		}

		$this->maxage = 86400;
		return $this->template->runtemplate('action_import_continuescreen');
	}

	function doimport($type, $frominput = false)
	{
		if (!$this->fullaccess) return $this->denyaccess('import identities');
		if (!isset($this->plugins[$type]))
		{
			$this->status = array(404, 'Not Found');
			$this->vars['error'] = 'unsupportedimporttype';
			$this->template->vars['notices'][] = "Error: the import source you chose (importsource_$type) is not currently installed.";
			return $this->showform();
		}

		$expectsfile = $this->plugins[$type]['expectsfile'];

		if ($expectsfile && !$frominput
			&& !is_uploaded_file($_FILES['importfile']['tmp_name']))
		{
			$this->status = array(400, 'Bad Request');
			$this->vars['error'] = 'fileuploadfailed';
			$this->template->vars['notices'][] = "Error: no file was uploaded, or there was a problem with the file upload (such as file size too large).";
			return $this->docontinue($type);
		}

		$filename = $expectsfile 
			? ($frominput ? 'php://input'	: $_FILES['importfile']['tmp_name'])
			: null;

		require_once(RESOURCE_DIR . "import/importsource_$type.inc.php");
		$sourcetype = "importsource_$type";
		$source = new $sourcetype($filename);

		if (!$source->isvalid())
		{
			$this->status = array(400, 'Bad Request');
			$this->vars['error'] = 'invalidimportdata';
			$this->template->vars['notices'][] = "Error: the file you uploaded does not appear to be valid.  Please ensure that you have selected the correct import type.";
			return $this->docontinue($type);
		}

		$importidentities = $this->controls->getcontrol('importidentities');

		if ($this->format == 'html')
		{
			$this->startrealtime();
			$this->template->vars['pagetitle'] = 'Processing identities...';
			$this->outputrealtime($this->template->runtemplate('progresswin_head'));
			$this->outputrealtime($this->template->runtemplate('action_import_start'));
			$importidentities->setprogresscallback(array($this, 'updateprogress'));
		}

		$status = $importidentities->import($source);
		$this->template->vars['action_status'] = $status;
		$this->vars['message'] = 'importsuccess';
		$this->vars['importedcount'] = $status['importedcount'];
		$this->vars['newcount'] = $status['newcount'];
		if (!empty($status['lastidentity']))
		{
			$this->vars['lastimportedid'] = $status['lastidentity']->id;
		}

		if ($this->format == 'html')
		{
			$this->outputrealtime($this->template->runtemplate('action_import_end'));
			$this->outputrealtime($this->template->runtemplate('progresswin_foot'));
		}
	}

	function updateprogress($progress)
	{
		$this->template->vars['action_progress'] = $progress;
		$this->outputrealtime($this->template->runtemplate('action_import_updateprogress'));
	}

	function showform()
	{
		if (!$this->fullaccess) return $this->denyaccess('import identities');
		$this->maxage = 86400;
		$this->vary = 'Cookie';
		if ($this->format == 'html')
			return $this->template->runtemplate('action_import');
	}

	function getimportplugins()
	{
		$dir = opendir(RESOURCE_DIR . 'import');
		if (!$dir) trigger_error('resources/import directory not found.', E_USER_ERROR);	

		$types = array();

		while (($file = readdir($dir)) !== false)
			if (preg_match('/^importsource_([\w-]+)\.inc\.php$/', $file, $matches))
		{
			$type = $matches[1];
			$classname = "importsource_$type";
			require_once(RESOURCE_DIR . "import/importsource_$type.inc.php");
			$desc = call_user_func(array($classname, "getname"));
			if ($desc === null) $desc = "[$type]";
			$types[$type] = array(
				'name' => $desc,
				'expectsfile' => call_user_func(array($classname, "expectsfile")),
			);
		}
		closedir($dir);

		$this->template->vars['action_importplugins'] = $vars['importplugins'] = $this->plugins = $types;
	}
}

?>
