<?php

/*
	action_new - for creating a new identity
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

class action_new extends module
{
	function display()
	{
		$this->template->vars['title'] = 'New identity';
		if (!$this->fullaccess) return $this->denyaccess('create new identities');

		$method = $this->restmethod();
		if ($method == 'POST') return $this->createfrominput();

		$affiliation = $this->context->load_var('affiliation', 'POST', 'name');
		$namestring = $this->context->load_var('name', 'POST', 'string');
		$nametype = $this->context->load_var('nametype', 'POST', 'name');

		if ($affiliation)
		{
			require_once(RESOURCE_DIR . 'includes/identity.inc.php');

			$identity = new identity();
			$identity->affiliation = $affiliation;
			$name = name::extract($namestring);
			$name->verified = true;
			if ($name->surname == '')
			{
				$this->template->vars['notices'][] = "Error: the name field was left blank, or did not appear to contain a name.  Please try again.";
				return $this->template->runtemplate('action_new');
			}
			$name->type = $nametype;
			$identity->names[] = $name;

			$editidentity = $this->controls->getcontrol('editidentity');
			$editidentity->updateidentity($identity);

			$identityid = $identity->id;
			if (!$identityid)
			{
				$this->status = array(500, 'Internal Server Error');
				$this->template->vars['notices'][] = "Error: there was an unknown problem creating this new identity.  Please check with your system administrator.";
				return $this->template->runtemplate('action_new');
			}

			$this->context->redirect(URL_PATH . "?a=edit&identityid=$identityid&f={$this->format}&m=createsuccess");
		}
		$this->maxage = 86400;
		if ($this->format == 'html')
			return $this->template->runtemplate('action_new');
	}

	function createfrominput()
	{
		if (!$this->fullaccess) return $this->denyaccess('create new identities');

		$identities = $this->getxmlidentities('php://input');

		if (!is_array($identities))
		{
			$this->status = array(400, 'Bad Request');
			$this->vars['error'] = 'invalidinput';
			$this->template->vars['notices'][] = 'The data submitted in the request body does not appear to be valid.';
			return $this->template->runtemplate('action_new');
		}

		$editidentity = $this->controls->getcontrol('editidentity');
		$fail = $success = 0;
		$lastidentityid = null;
		foreach ($identities as $identity)
		{
			$identity->anonymise();
			$identity->localise();

			$result = $editidentity->updateidentity($identity);
			if (!$result) $fail++;
			else
			{
				$success++;
				$lastidentityid = $identity->id;
			}
		}

		if ($fail > 0)
		{
			if ($success <= 0) $this->status = array(500, 'Internal Server Error');
			$this->vars['error'] = 'partialfailure';
			$this->vars['failcount'] = $fail;
			$this->template->vars['notices'][] = 'One or more new identities failed to be created due to an internal error.';
		}
		else
			$this->vars['message'] = 'createfullsuccess';
		$this->vars['successcount'] = $success;
		if ($lastidentityid !== null)
			$this->vars['lastinsertedid'] = $lastidentityid;
		
		if ($this->format == 'html')
			return $this->template->runtemplate('action_new');
	}
}

?>
