<?php

/*
	action_oaipmh - module for acting as an oai-pmh repository
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

class action_oaipmh extends module
{
	private
		$formats,
		$parms,
		$document;

	function display()
	{
		// the html display only returns a notice saying how to get to the feed.
		// all business for this action is done by the xml format
		$this->template->vars['title'] = 'OAI-PMH repository access';
		$this->template->vars['action_publicaddress'] = $this->getnicnamesurl();
		$this->template->vars['action_adminaddress'] = $this->getnicnamesurl() . 'admin/';

		if (isset($_POST['verb']) || isset($_GET['verb']))
		{
			$this->format = 'xml';
			return null;
		}
		else
		{
			return $this->template->runtemplate("action_oaipmh");
		}
	}

	function xmlformat() // override the inherited xmlformat
	{
		$this->document = new DomDocument();
		$rootnode = $this->document->createElement("OAI-PMH");
		$rootnode->setAttribute("xmlns", "http://www.openarchives.org/OAI/2.0/");
		$rootnode->setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		$rootnode->setAttribute("xsi:schemaLocation", "http://www.openarchives.org/OAI/2.0/ http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd");
		$this->document->appendChild($rootnode);

		$this->formats = array(
			'nicnames' => array($this->getnicnamesurl() . "nicnames.xsd",
				"http://swin.edu.au/lib/research/nicnames"),
			'oai_dc' => array("http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
				"http://www.openarchives.org/OAI/2.0/oai_dc/"),
			);

		$timenow = time();
		$rootnode->appendChild($this->document->createElement(
			"responseDate", gmdate("Y-m-d\TH:i:s\Z", $timenow)));

		$requestelement = $this->document->createElement(
			"request", htmlspecialchars($this->getbaseurl()));
		$rootnode->appendChild($requestelement); 

		$errors = $this->getparms();

		if (!$errors)
		{
			switch ($this->parms['verb'])
			{
			case "ListIdentifiers":
				$response = $this->dolistrecords(true);
				break;
			case "ListRecords":
				$response = $this->dolistrecords();
				break;
			case "GetRecord":
				$response = $this->dogetrecord();
				break;
			case "Identify":
				$response = $this->doidentify();
				break;
			case "ListMetadataFormats":
				$response = $this->dolistmetadataformats();
				break;
			case "ListSets":
				$response = $this->geterror("noSetHierarchy", "This implementation does not support sets.");
				//$response = $this->dolistsets();
				break;
			}
			$rootnode->appendChild($response);
			if ($response->nodeName != 'error')
			{
				foreach ($this->parms as $name => $val) if ($val !== null)
				{
					$requestelement->setAttribute($name, htmlspecialchars($val));
				}
			}
		}
		else
		{
			foreach ($errors as $error) $rootnode->appendChild($error);
		}

		return $this->document->saveXML();
	}

	private function getparms()
		// sets this->parms to an array of fetched arguments.  returns any errors
	{
		$parmlist = array(
			'GetRecord' => array('identifier','metadataPrefix'),
			'Identify' => array(),
			'ListIdentifiers' => array('from','until','metadataPrefix','set','resumptionToken'),
			'ListMetadataFormats' => array('identifier'),
			'ListRecords' => array('from','until','metadataPrefix','set','resumptionToken'),
			'ListSets' => array('resumptionToken'),
			);
		$verb = null;
		$this->parms = array();
		$errors = array();

		if (isset($_SERVER['QUERY_STRING']))
		{
			$qsparms = explode('&', $_SERVER['QUERY_STRING']);
			$qsvars = array();
			foreach ($qsparms as $parm) if ($parm[0] != '=')
			{
				list($name) = explode('=', $parm);
				if (isset($qsvars[$parm]))
					$errors[] = $this->geterror("badArgument", "Same argument appears twice in query string.");
				$qsvars[$parm] = 1;
			}
		}

		if (isset($_POST['verb'])) $verb = $_POST['verb'];
		else if (isset($_GET['verb'])) $verb = $_GET['verb'];
		else $errors[] = $this->geterror("badVerb", "Argument 'verb' is missing.");

		if ($verb !== null && !isset($parmlist[$verb]))
			$errors[] = $this->geterror("badVerb", "Argument 'verb' does not refer to a known verb.");

		$validparms = isset($parmlist[$verb]) ? $parmlist[$verb] : array();

		foreach ($_POST as $name => $val)
		{
			if ($name != 'verb' && !in_array($name, $validparms))
				$errors[] = $this->geterror("badArgument", "Argument '$name' is not allowed.");
			$this->parms[$name] = (string)$val;
		}
		foreach ($_GET as $name => $val)
		{
			if (isset($this->parms[$name]) && $this->parms[$name] !== null)
				$errors[] = $this->geterror("badArgument", "Argument '$name' was supplied more than once in GET and POST.");
			if ($name != 'verb' && !in_array($name, $validparms))
				$errors[] = $this->geterror("badArgument", "Argument '$name' is not allowed.");
			$this->parms[$name] = (string)$val;
		}

		foreach ($this->parms as $name => $val) if ($val !== null) 
		{
			if ($name == 'metadataPrefix' && !isset($this->formats[$val]))
			{
				$errors[] = $this->geterror("cannotDisseminateFormat", "Argument 'metadataPrefix' refers to an unknown format.");
				$errors[] = $this->geterror("badArgument", "Bad value for argument 'metadataPrefix'.");
			}
			if (($name == 'from' || $name == 'until') && !preg_match('#^\d\d\d\d-\d\d-\d\d(T\d\d:\d\d:\d\dZ)?$#', $val))
				$errors[] = $this->geterror('badArgument', "Bad value for argument '$name'.");

			if ($name == 'from' && !empty($this->parms['until']) && strlen($val) != strlen($this->parms['until']))
				$errors[] = $this->geterror('badArgument', "Arguments 'from' and 'until' must be the same length (have same granularity).");
		}

		foreach ($validparms as $parm) if (!isset($this->parms[$parm])) 
			$this->parms[$parm] = null;

		return $errors;
	}

	private function dolistrecords($headersonly = false)
	{
		$from = $this->parms['from'];
		$until = $this->parms['until'];
		$set = $this->parms['set'];
		$resumetoken = $this->parms['resumptionToken'];
		$metadataformat = $this->parms['metadataPrefix'];

		if ($resumetoken !== null && ($from !== null || $until !== null || $set !== null || $metadataformat !== null))
			return $this->geterror("badArgument", "Cannot specify both a resumption token and either from, until, set or metadataPrefix.");

		if ($from !== null)
		{
			$from = strtotime($from);
			if (!$from) return $this->geterror("badArgument", "Bad from argument.");
		}
		else $from = 0;

		if ($until !== null)
		{
			$until = strtotime($until);
			if (!$until) return $this->geterror("badArgument", "Bad until argument.");
		}

		if ($set !== null)
			return $this->geterror("noSetHierarchy", "This implementation does not support sets");

		$skipids = array();
		if ($resumetoken !== null)
		{
			if (!preg_match("#^\d+-(\d+|null)-\w+(-\d+)*$#", $resumetoken))
				return $this->geterror("badResumptionToken", "The supplied resumptionToken was invalid.");
			$resumptiondetails = explode('-', $resumetoken);
			if ($resumptiondetails[1] < $resumptiondetails[0])
				return $this->geterror("badResumptionToken", "The supplied resumptionToken is not valid for the given request.");
			$from = array_shift($resumptiondetails);
			$until = array_shift($resumptiondetails);
			if ($until == 'null') $until = null;
			$metadataformat = array_shift($resumptiondetails);
			$skipids = $resumptiondetails;
		}

		if ($metadataformat === null)
			return $this->geterror("badArgument", "Missing metadataPrefix argument.");

		$fetchidentities = $this->controls->getcontrol('fetchidentities');
		$ids = $fetchidentities->getidsmodifiedsince($from, 1001+count($skipids), $until);

		$identities = $fetchidentities->getidentities($ids);
		if (!count($identities))
			return $this->geterror("noRecordsMatch", "The given request resulted in no matches.");

		$element = $this->document->createElement($headersonly ? "ListIdentifiers" : "ListRecords");

		$count = 0;
		$overflowed = false;
		$perdate = array();

		foreach ($identities as $identity)
		{
			if (!$this->fullaccess) $identity->stripprivate();

			if ($count == 1000)
			{
				$resumedetails = array(
					$identity->lastmodifieddate,
					$until === null ? 'null' : $until,
					$metadataformat);
				if (isset($perdate[$identity->lastmodifieddate]))
					$resumedetails = array_merge($resumedetails, $perdate[$identity->lastmodifieddate]);
				$resumetoken = implode('-', $resumedetails);
				$element->appendChild($this->document->createElement("resumptionToken", htmlspecialchars($resumetoken)));
				$overflowed = true;
				break;
			}
			$perdate[$identity->lastmodifieddate][] = $identity->id;
			if (!in_array($identity->id, $skipids))
			{
				$record = $headersonly ? $this->getheader($identity) : $this->getrecord($identity, $metadataformat);
				if ($record->nodeName == "error") return $record;
				$element->appendChild($record);
				$count++;
			}
		}

		if ($resumetoken && !$overflowed)
		{
			// add empty resumption token to incomplete list which has finished normally
			$element->appendChild($this->document->createElement("resumptionToken"));
		}

		return $element;
	}

	private function dogetrecord()
	{
		$id = (int)$this->parms['identifier'];
		if (!$id) return $this->geterror("badArgument", "Missing identifier argument, or identifier argument is not an integer.");
		$metadataformat = $this->parms['metadataPrefix'];
		if ($metadataformat === null)
			return $this->geterror("badArgument", "Missing metadataPrefix argument.");

		$fetchidentities = $this->controls->getcontrol('fetchidentities');
		$identities = $fetchidentities->getidentities($id);
		if (empty($identities))
			return $this->geterror("idDoesNotExist", "An identity with the given ID could not be found.");

		$identity = reset($identities);

		if (!$this->fullaccess)
			$identity->stripprivate();

		$response = $this->document->createElement("GetRecord");
		$record = $this->getrecord($identity, $metadataformat);

		if ($record->nodeName == 'error') return $record;

		$response->appendChild($record);
		return $response;
	}

	private function getheader($identity)
	// gets just the OAI-PMH header of this identity.  works whether $identity is
		// an actual identity or a deletion (stub), but it must have an ID and modification
		// date
	{
		$header = $this->document->createElement("header");

		$header->appendChild($this->document->createElement("identifier", (int)$identity->id));
		$header->appendChild($this->document->createElement("datestamp", gmdate("Y-m-d\TH:i:s\Z", $identity->lastmodifieddate)));

		if ($identity instanceof identity_alias)
			$header->setAttribute("status", "deleted");

		return $header;
	}

	private function getrecord($identity, $metadataformat)
	// gets the record, including header, and metadata.
		// the metadata is omitted if it's a deletion
	{
		$record = $this->document->createElement("record");
		$record->appendChild($this->getheader($identity));

		if ($identity instanceof identity)
		{
			// in OAI-PMH a record doesn't have any metadata element
			// if it is a deleted record (which are identity_alias class instead)
			$metadata = $this->document->createElement("metadata");
			$element = null;
			switch ($metadataformat)
			{
			case 'oai_dc':
				$element = $identity->getOAIDC($this->document);
				$element->setAttribute("xmlns:oai_dc", $this->formats['oai_dc'][1]);
				$element->setAttribute("xsi:schemaLocation", $this->formats['oai_dc'][1] . ' ' .$this->formats['oai_dc'][0]);
				break;
			case 'nicnames':
				$element = $identity->getXML($this->document);
				$element->setAttribute("xmlns", $this->formats['nicnames'][1]);
				$element->setAttribute("xsi:schemaLocation", $this->formats['nicnames'][1] . ' ' .$this->formats['nicnames'][0]);
				break;
			default:
				return $this->geterror("cannotDisseminateFormat", "The requested metadata format is not supported.  Please try the \"nicnames\" or the \"oai_dc\" format.");
			}

			$metadata->appendChild($element);
			$record->appendChild($metadata);
		}
		
		return $record;
	}

	private function doidentify()
	{
		$response = $this->document->createElement("Identify");
		$response->appendChild($this->document->createElement("repositoryName", "NicNames"));
		$response->appendChild($this->document->createElement("baseURL", htmlspecialchars($this->getbaseurl())));
		$response->appendChild($this->document->createElement("protocolVersion", "2.0"));
		$response->appendChild($this->document->createElement("adminEmail", htmlspecialchars($this->getadminemail())));
		$response->appendChild($this->document->createElement("earliestDatestamp", "2001-01-01T01:00:00Z"));
		$response->appendChild($this->document->createElement("deletedRecord", "persistent"));
		$response->appendChild($this->document->createElement("granularity", "YYYY-MM-DDThh:mm:ssZ"));
		$response->appendChild($this->document->createComment("Note: this implementation does not record an administrator email address.  The value of adminEmail may not be a real address."));

		return $response;
	}

	private function dolistmetadataformats()
	{
		$id = $this->parms['identifier'];
		if ($id !== null)
		{
			$fetchidentities = $this->controls->getcontrol('fetchidentities');
			$identities = $fetchidentities->getidentities((int)$id);
			if (empty($identities))
				return $this->geterror("idDoesNotExist", "The given record ID does not exist.");
		}

		$response = $this->document->createElement("ListMetadataFormats");

		foreach ($this->formats as $prefix => $val)
		{
			$el = $this->document->createElement("metadataFormat");
			$el->appendChild($this->document->createElement("metadataPrefix", $prefix));
			$el->appendChild($this->document->createElement("schema", htmlspecialchars($val[0])));
			$el->appendChild($this->document->createElement("metadataNamespace", htmlspecialchars($val[1])));
			$response->appendChild($el);
		}

		return $response;
	}

	/*
	private function dolistsets()
	{
		$response = $this->document->createElement("ListSets");

		$sets = array(
			'affiliated' => 
			);

		foreach ($formats as $prefix => $val)
		{
			$el = $this->document->createElement("metadataFormat");
			$el->appendChild($this->document->createElement("metadataPrefix", $prefix));
			$el->appendChild($this->document->createElement("schema", htmlspecialchars($val[0])));
			$el->appendChild($this->document->createElement("metadataNamespace", htmlspecialchars($val[1])));
			$response->appendChild($el);
		}

		return $response;
	}
	 */

	private function getadminemail()
	{
		if (empty($_SERVER['SERVER_NAME'])) return "";
		return "postmaster@" . $_SERVER['SERVER_NAME'];
	}

	private function geterror($type, $description = null)
	{
		$error = $this->document->createElement("error", htmlspecialchars($description));
		$error->setAttribute("code", $type);
		return $error;
	}



	/*
	function display_old()
	{
		$m = $this->context->load_var('m', 'REQUEST', 'name');
		if ($m == 'deleteidentitysuccess')
			$this->template->vars['notices'][] = 'The identity you chose was successfully deleted.';
		if ($m) $this->vars['message'] = $m;

		$queryperformed = false;
		$searchurl = '?a=query';
		$searchdesk = null;

		$time = $this->context->load_var("modifiedsince", 'REQUEST', 'string');
		if ($time)
		{
			$cutoff = (int)strtotime($time);
			$fetchidentities = $this->controls->getcontrol('fetchidentities');
			$ids = $fetchidentities->getidsmodifiedsince($cutoff);
			$this->identities = $this->layoutidentities($ids);
			$queryperformed = true;
			$searchurl = "?a=query&modifiedsince=" . urlencode(gmdate("Y-m-d\TH:i:s\Z", $cutoff));
			$searchdesc = "records modified since " . date('j M Y', $cutoff);
			$timeofday = date('H:i:s', $cutoff);
			if ($timeofday != '00:00:00') $searchdesc .= " ($timeofday)";
		}
		else
		{
			$this->identities = $this->findidentities();
			$queryperformed = $this->querystring != '';
			$searchurl = "?a=query&query=" . urlencode($this->querystring);
			$searchdesc = $this->querystring;
		}

		$this->template->vars['title'] = 'Search';
		$this->template->vars['action_queryperformed'] = $queryperformed;
		$this->template->vars['action_searchurl'] = $searchurl;
		$this->template->vars['action_searchdesc'] = $searchdesc; 

		// client-side cache up to 15 minutes for public access, 30 seconds for admin
		if (!$time) $this->maxage = $this->fullaccess ? 30 : 900;
		$this->vary = 'Cookie';
		if (!$queryperformed) $this->maxage = 86400;

		if ($this->format == 'html')
			return $this->template->runtemplate('action_query');
	}
	 */
}

?>
