<?php

/*
	nicnamesinterface - Main interface for NicNames
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
	
*/

/*

	This is the main controller for NicNames - it initiates the environment,
	including
	
	- context object (for input/output)
	- debugging object
	- database object (very simple DB abstraction)
	- controls object (for loading modules of the 'model')
	
	Every page viewed has an 'action' (the default is the 'query' action)
	Actions are modular, to define a new one just add it to resources/modules
	
*/

require_once(BLUESTONE_DIR . 'debug.inc.php');
require_once(BLUESTONE_DIR . 'context.inc.php');
require_once(BLUESTONE_DIR . 'db_connection.inc.php');
require_once(RESOURCE_DIR . 'includes/controls.inc.php');
require_once(RESOURCE_DIR . 'includes/templateengine.inc.php');
require_once(RESOURCE_DIR . 'includes/module.inc.php');

class nicnamesinterface
{
	function nicnamesinterface($fullaccess = false)
	{
		$this->fullaccess = (bool)$fullaccess;

		// set up environment
		$this->debug = debug::getinstance(DEBUG);
		$this->debug->useerrorhandler();
		$this->context = context::getinstance();
		$this->debug->seterrorcallback(array(&$this->context, 'fatal_error'));
		
		$this->db = new db_connection(array(
			'persistent' => false,
			'server' => DB_HOSTNAME,
			'database' => DB_DATABASE,
			'user' => DB_USERNAME,
			'pass' => DB_PASSWORD,
			'prefix' => DB_TABLEPREFIX,
			'charset' => 'utf8',
			));
		
		$controls = &controls::getinstance();
		$controls->setproperties($this->db);
		
		$this->template = new templateengine();
		
		$this->renderpage();
	}
	
	function renderpage()
	// figures out the page type, runs the right code and shows the page
	// (controller)
	{
		$action = $this->context->load_var('a', 'REQUEST', 'name');
		if (!$action)
			// detect OAI-PMH request which has 'verb' as a POST or GET
			$action = (isset($_POST['verb']) || isset($_GET['verb'])) ?
				'oaipmh' : 'query';

		if ($action != 'install' && !$this->db->is_connected())
			trigger_error('MySQL connection failed, please check configuration file; '. $this->db->get_error(), E_USER_ERROR);
		$this->template->vars['action'] = $action;
		$this->template->vars['site_root_path'] = $this->fullaccess ? '../' : '';
		$this->template->vars['fullaccess'] = $this->fullaccess;
		$this->template->vars['query_string'] = $_SERVER['QUERY_STRING'];

		$format = $this->context->load_var('f', 'REQUEST', 'name');
		if ($format === null)
			$format = 'html';
	
		if (!preg_match('/^[\w.-]+$/', $action))
			trigger_error("Invalid module name '$action'", E_USER_ERROR);
		$classname = "action_$action";
		require_once(RESOURCE_DIR . "modules/$classname.inc.php");
		
		$this->module = new $classname($this->template, $format, $this->fullaccess);
		$this->module->fullaccess = $this->fullaccess;
		$data = $this->module->display();
		$fname = "NicNames "
			. preg_replace('#["\x0d\x80-\xff\\\\]#', '', $this->template->vars['title'])
			. ' ' . date("Y-m-d his", TIMENOW);

		if ($this->module->format) // modules can change their desired format
			$format = $this->module->format;

		switch ($format) 
		{
		case 'html':
			$this->template->vars['contentarea_html'] = $data;
			$data = $this->template->runtemplate('main');
			break;
		case 'xml':
			$data = $this->module->xmlformat();
			// hack - oai-pmh must use text/xml not application/xml
			$this->context->setcontenttype($action=='oaipmh' ? 'text/xml' : 'application/xml');
			$this->context->header("Content-disposition: inline; filename=\"$fname.xml\"");
			break;
		case 'json':
			$data = $this->module->jsonformat();
			$this->context->setcontenttype('application/json');
			$this->context->header("Content-disposition: inline; filename=\"$fname.json\"");
			break;
		}

		if (!$this->module->realtime)
		{
			if ($this->module->maxage !== null)	$this->context->setmaxage($this->module->maxage);
			if ($this->module->vary !== null) $this->context->setvary($this->module->vary);
			if ($this->module->status !== null)
				$this->context->setstatus($this->module->status[0], $this->module->status[1]);
			$this->context->dooutput($data);
		}
	}
	
}

?>
