<?php

require_once(dirname(__FILE__) . '/../resources/global.inc.php');
require_once(RESOURCE_DIR . '../tests/libraries/simpletest/autorun.php');

require_once(RESOURCE_DIR . 'includes/identity.inc.php');

class identity_test extends UnitTestCase
{
	function testfromxml()
	{
		$doc = new DomDocument();
		$doc->load(RESOURCE_DIR . 'import/sample.xml');

		$identity = new identity();

		$this->assertTrue($identity);
		$this->assertTrue($identity->names === array());

		$element = $doc->getElementsByTagName('identity')->item(0);
		$identity->setfromXML($element);

		$this->assertTrue($identity->names !== array());
		$this->assertTrue($identity->names[0]->surname != '');
		$this->assertTrue($identity->birthyear);
	}

	function testtoxml()
	{
		$identity = new identity();
		$identity->id = 89766456;
		$name = new name();
		$name->surname = 'Rockerfeller';
		$name->type = "publishedasname";
		$identity->names[] = $name;

		$doc = new DomDocument();
		$rootnode = $doc->createElement('nicnames');
		$rootnode->setAttribute('xmlns', 'http://swin.edu.au/lib/research/nicnames');
		$doc->appendChild($rootnode);
		$el = $identity->getXML($doc);
		$rootnode->appendChild($el);
		$xmltext = $doc->saveXML();

		$this->assertTrue(strlen($xmltext) > 10);
		$this->assertTrue(strpos($xmltext, '89766456') !== false);
		$this->assertTrue(strpos($xmltext, 'Rockerfeller') !== false);

		$doc2 = new DomDocument();
		$doc2->loadXML($xmltext);

		$result = $doc2->schemavalidate(RESOURCE_DIR . 'import/nicnames.xsd');

		$this->assertTrue($result);
	}

	function testsamplevalidate()
	{
		$doc = new DomDocument();
		$doc->load(RESOURCE_DIR . 'import/sample.xml');
		$result = $doc->schemaValidate(RESOURCE_DIR . 'import/nicnames.xsd');

		$this->assertTrue($result);
	}

	function testcomparenames()
	{
		$name1 = name::extract("Troy, Thomas Albert stewart");
		$name2 = name::extract("Rutter, Ralph Albert");
		$name3 = name::extract("Rutter, Ralph A. W.");
		$name4 = name::extract("Pickering, Ralph Albert");

		$this->assertTrue($name1->similarityto($name2) < 0.4);
		$this->assertTrue($name2->similarityto($name3) > 0.75);
		$this->assertTrue($name4->similarityto($name2) < 0.4);
		$this->assertTrue($name1->similarityto($name1) > 0.97);

		$name1 = name::extract("Karen Michelle Babanin");
		$name2 = name::extract("Karen Michelle Kaltmann");
		$name3 = name::extract("K. M. L. Babanin");
		$name4 = name::extract("George Michael Babanin");
		$name5 = name::extract("Madonna");

		$this->assertTrue($name1->similarityto($name2) < 0.5);
		$this->assertTrue($name1->similarityto($name3) > 0.75);
		$this->assertTrue($name1->similarityto($name4) < 0.5);
		$this->assertTrue($name5->similarityto($name5) > 0.97);

		$name1 = name::extract("Jones, John R.");
		$name2 = name::extract("Thinn, Moe Moe");
		$name3 = name::extract("Thien, Hla Hla");

		$this->assertTrue($name1->similarityto($name1) > 0.97);
		$this->assertTrue($name2->similarityto($name2) > 0.97);
		$this->assertTrue($name3->similarityto($name3) > 0.97);
	}

	function testcompareclassifications()
	{
		$classification1 = new classification();
		$classification2 = new classification();
		$classification3 = new classification();

		$classification1->code = "A99003";
		$classification2->code = "A99003";
		$classification2->text = "Computer Science";
		$classification3->text = "Computer science";
		
		$this->assertTrue($classification1->similarityto($classification2) > 0.5);
		$this->assertTrue($classification1->similarityto($classification3) < 0.4);
		$this->assertTrue($classification2->similarityto($classification3) > 0.1);
		$this->assertTrue($classification1->similarityto($classification1) > 0.97);
		$this->assertTrue($classification2->similarityto($classification2) > 0.97);
		$this->assertTrue($classification3->similarityto($classification3) > 0.97);
	}

	function testcompareevents()
	{
		$event1 = new event();
		$event2 = new event();
		$event3 = new event();

		$event1->title = "Conference for computer science";
		$event1->type = "conference";
		$event2->title = "conference, Computer science";
		$event2->type = "conference";
		$event3->title = "Conference for science is our future";
		$event3->type = "parade";
		
		$this->assertTrue($event1->similarityto($event2) > 0.5);
		$this->assertTrue($event2->similarityto($event3) < 0.5);
		$this->assertTrue($event1->similarityto($event1) > 0.97);
		$this->assertTrue($event3->similarityto($event3) > 0.97);
	}

	function testcompareidentifiers()
	{
		$identifier1 = new identifier();
		$identifier2 = new identifier();
		$identifier3 = new identifier();

		$identifier1->identifier = "A990334";
		$identifier1->type = "staffid";
		$identifier2->identifier = "A990334";
		$identifier2->type = "staffid";
		$identifier3->identifier = "a403935";
		$identifier3->type = "staffid";
		
		$this->assertTrue($identifier1->similarityto($identifier2) > 0.9);
		$this->assertTrue($identifier2->similarityto($identifier3) < 0.5);
		$this->assertTrue($identifier1->similarityto($identifier1) > 0.97);
		$this->assertTrue($identifier3->similarityto($identifier3) > 0.97);
	}

	function testcompareorganisations()
	{
		$organisation1 = new organisation();
		$organisation2 = new organisation();
		$organisation3 = new organisation();

		$organisation1->title = "Swinburne university of technollogy";
		$organisation1->type = "institution";
		$organisation2->title = "Swinburne University of Technology";
		$organisation2->type = "institution";
		$organisation3->title = "New England College of Medicine";
		$organisation3->type = "institution";
		
		$this->assertTrue($organisation1->similarityto($organisation2) > 0.9);
		$this->assertTrue($organisation2->similarityto($organisation3) < 0.5);
		$this->assertTrue($organisation1->similarityto($organisation1) > 0.97);
		$this->assertTrue($organisation3->similarityto($organisation3) > 0.97);
	}
	
	function testcomparepersons()
	{
		$person1 = new person();
		$person2 = new person();
		$person3 = new person();

		$person1->person = new identity();
		$person1->person->names[] = new name();
		$person1->person->names[0]->surname = "Smith";
		$person1->person->names[0]->givennames = "Tony";
		$person2->person = new identity();
		$person2->person->names[] = new name();
		$person2->person->names[0]->surname = "Smith";
		$person2->person->names[0]->givennames = "t. s.";
		$person3->person = new identity();
		$person3->person->names[] = new name();
		$person3->person->names[0]->surname = "Jones";
		$person3->person->names[0]->givennames = "ralph";

		$this->assertTrue($person1->similarityto($person2) > 0.5);
		$this->assertTrue($person2->similarityto($person3) < 0.5);
		$this->assertTrue($person1->similarityto($person1) > 0.97);
		$this->assertTrue($person2->similarityto($person2) > 0.97);
		$this->assertTrue($person3->similarityto($person3) > 0.97);
	}
	
	function testcompareresources()
	{
		$resource1 = new resource();
		$resource2 = new resource();
		$resource3 = new resource();

		$resource1->title = "journle of medicine";
		$resource1->type = "journal";
		$resource2->title = "Journal of Medicine";
		$resource2->type = "journal";
		$resource3->title = "Information Resources : The definitive guide";
		$resource3->type = "book";
		
		$this->assertTrue($resource1->similarityto($resource2) > 0.9);
		$this->assertTrue($resource2->similarityto($resource3) < 0.4);
		$this->assertTrue($resource1->similarityto($resource1) > 0.97);
		$this->assertTrue($resource3->similarityto($resource3) > 0.97);
	}

	function testcompareidentities()
	{
		$identity1 = new identity();
		$identity2 = new identity();
		$identity3 = new identity();

		$name1 = new name();
		$name1->type = 'publishedasname';
		$name1->surname = 'smith';
		$name1->givennames = 'tony jones';
		$identity1->names[] = $name1;

		$name2 = new name();
		$name2->type = 'publishedasname';
		$name2->surname = 'roberts';
		$name2->givennames = 'julia louise';
		$identity2->names[] = $name2;

		$name3 = new name();
		$name3->type = 'birthname';
		$name3->surname = 'smith';
		$name3->givennames = 'Tony J. L.';
		$identity3->names[] = $name3;

		$this->assertTrue($identity3->similarityto($identity1) > 0.5);
		$this->assertTrue($identity3->similarityto($identity1) > $identity3->similarityto($identity2));
		$this->assertTrue($identity1->similarityto($identity1) > 0.75); // only one name match

		$results = $identity3->findmatch(array(1 => $identity1, 2 => $identity2));
		$this->assertEqual($results['foundmatch'], 1);
		$this->assertTrue($results['bestscore'] > 0.5);
		$this->assertEqual($results['uncertain'], 1); // should be uncertain as only one name match
		$this->assertEqual($results['othercandidates'], 0);
		//print_r($results);
	}

	function testgettextfields()
	{
		$identity = new identity();
		$identity->id = 89766456;
		$name = new name();
		$name->surname = 'Rockerfeller';
		$name->type = 'publishedasname';
		$identity->names[] = $name;

		$groups = $identity->gettextfieldgroups();
		$this->assertTrue(!empty($groups));
		$this->assertTrue(!empty($groups['name']));
		$this->assertTrue(!empty($groups['identity']));
		$this->assertTrue(is_numeric($groups['name'][0][1]));
		$this->assertTrue(strpos($groups['name'][0][0], 'Rockerfeller') !== false);
	}

	function testmergerelation()
	{
		$name1 = new name();
		$name2 = new name();

		$name1->surname = "Smith";
		$name1->givennames = "Thomas";
		$name1->incidence = 1;
		$name1->startyear = 1980;
		$name1->endyear = 2007;

		$name2->surname = "smith";
		$name2->title = "Mr";
		$name2->incidence = 3;
		$name2->startyear = 1981;
		$name2->endyear = 2009;

		$name1->merge($name2);

		$this->assertEqual($name1->surname, "Smith");
		$this->assertEqual($name1->givennames, "Thomas");
		$this->assertEqual($name1->incidence, 3); // larger incidence should win
		$this->assertEqual($name1->startyear, 1980); // earlier startdate should win
		$this->assertEqual($name1->endyear, 2009); // later endyear should win
		$this->assertEqual($name1->title, "Mr"); // more 'filled in' should win
	}

	function testmergeidentity()
	{
		$identity1 = new identity();
		$identity2 = new identity();

		$identity1->id = 89766456;
		$name = new name();
		$name->surname = 'Rockerfeller';
		$identity1->names[] = $name;
		$resource = new resource();
		$resource->title = "The book of five";
		$identity1->resources[] = $resource;

		$identity2->id = 249568;
		$name = new name();
		$name->surname = 'Davies';
		$identity2->names[] = $name;
		$resource = new resource();
		$resource->title = "The Book of Five";
		$identity2->resources[] = $resource;

		$identity1->merge($identity2);

		$this->assertEqual($identity1->id, 89766456);
		$this->assertEqual(count($identity1->names), 2); // different facts should accumulate
		$this->assertEqual($identity1->names[0]->surname, "Rockerfeller");
		$this->assertEqual($identity1->names[1]->surname, "Davies");
		$this->assertEqual(count($identity1->resources), 1); // substancially similar facts should be merged
		$this->assertEqual($identity1->resources[0]->title, "The book of five");
	}

}

?>
