<?php

require_once(dirname(__FILE__) . '/../resources/global.inc.php');
require_once(RESOURCE_DIR . '../tests/libraries/simpletest/autorun.php');

require_once(RESOURCE_DIR . 'includes/textfuzzy.inc.php');

class textfuzzy_test extends UnitTestCase
{
	function testgetphon()
	{
		$fuzzy1 = new textfuzzy("machine");

		$phon1 = $fuzzy1->getphon();

		// has output
		$this->assertTrue(is_string($phon1));
		$this->assertNotEqual($phon1, '');

		// test a non-match
		$fuzzy2 = new textfuzzy("verbatim");
		$this->assertTrue($phon1 != $fuzzy2->getphon());

		$fuzzy1 = new textfuzzy("more than one word");
		$this->assertTrue(preg_match('/\w+ \w+ \w+ \w+/', $fuzzy1->getphon()));

		$testcomparisons = array(
			'one' => 'won',
			'their' => 'there',
			'there' => 'they\'re',
			'two words' => 'too words',
			'"has, punctuation"' => 'has punctuation',
			"has    extra\tspaces" => 'has extra spaces',
			'would' => 'wood',
			'rebecca' => 'rebekah',
			'color' => 'colour',
			'initialisation' => 'initialization',
			'que' => 'queue',
			'light' => 'lite',
			'shaun' => 'sean',
			'swinburne' => 'swinburn',
			'metre' => 'meter',
			'ladel' => 'ladle',
			'know' => 'no',
			'five' => '5',
		);

		foreach ($testcomparisons as $string1 => $string2)
		{
			$fuzzy1 = new textfuzzy($string1);
			$fuzzy2 = new textfuzzy($string2);
			$this->assertNotEqual($fuzzy1->getphon(), '');
			$this->assertIdentical($fuzzy1->getphon(), $fuzzy2->getphon());
		}
	}

	function testsimilarity()
	{
		// test sameness
		$fuzzy1 = new textfuzzy("Relation");
		$fuzzy2 = new textfuzzy("Relation");
		$this->assertEqual($fuzzy1->similarityto($fuzzy2), 1.0);
		 
		// test very similar but not same
		$fuzzy1 = new textfuzzy("Relation");
		$fuzzy2 = new textfuzzy("Relasion");
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > 0.7);
		$this->assertNotEqual($fuzzy1->similarityto($fuzzy2), 1.0);

		// test somewhat similar in pronunciation
		$fuzzy1 = new textfuzzy("The third of November");
		$fuzzy2 = new textfuzzy("Ta fird of Now vemba");
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > 0.5);
		$this->assertNotEqual($fuzzy1->similarityto($fuzzy2), 1.0);

		// test very different
		$fuzzy1 = new textfuzzy("Relation");
		$fuzzy2 = new textfuzzy("Ballpoint");
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) < 0.1);

		// test both empty strings
		$fuzzy1 = new textfuzzy('');
		$fuzzy2 = new textfuzzy('');
		$this->assertIdentical($fuzzy1->similarityto($fuzzy2), null);

		// test relative sameness
		$fuzzy1 = new textfuzzy("Relation");
		$fuzzy2 = new textfuzzy("Ralashione");
		$fuzzy3 = new textfuzzy("Ralish");
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > $fuzzy1->similarityto($fuzzy3));
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > $fuzzy2->similarityto($fuzzy3));

		// test longer strings
		$fuzzy1 = new textfuzzy("The quick brown fox jumps over the lazy dog");
		$fuzzy2 = new textfuzzy("tha kwikbroun fux jumpova tha lazzi dog alot!!!!!");
		$fuzzy3 = new textfuzzy("The quick brown fox slips under my old bird");
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > $fuzzy1->similarityto($fuzzy3));
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > $fuzzy2->similarityto($fuzzy3));

		// test reversal
		$fuzzy1 = new textfuzzy("Center for esteemed excellence, The");
		$fuzzy2 = new textfuzzy("The new center for esteemed excellence Inc.");
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > 0.5);

		// combination test
		$fuzzy1 = new textfuzzy("Center for esteemed excellence");
		$fuzzy2 = new textfuzzy("sentre four esteemed eksellance");
		$this->assertTrue($fuzzy1->similarityto($fuzzy2) > 0.5);
	}

	function testgetphonkeys()
	{
		$fuzzy = new textfuzzy("quick brown fox kwik");
		$keys = $fuzzy->getphonkeys();
		$this->assertTrue(count($keys) == 4);
		$this->assertEqual($keys[0], $keys[3]);
		$this->assertNotEqual($keys[0], $keys[1]);
		$this->assertNotEqual($keys[1], $keys[2]);
	}

	function testgetcommonwordscores()
	{
		$fuzzy = new textfuzzy("elephant is of at and a stationery");
		$scores = $fuzzy->getcommonwordscores();

		$this->assertEqual($scores[0], 1);
		$this->assertEqual($scores[6], 1);
		$this->assertTrue($scores[1] < 0.99);
		$this->assertTrue($scores[2] < 0.99);
		$this->assertTrue($scores[3] < 0.99);
		$this->assertTrue($scores[4] < 0.99);
		$this->assertTrue($scores[5] < 0.99);
		foreach ($scores as $score)
			$this->assertTrue($score > 0.01);
	}

	function testgetstemmed()
	{
		$fuzzy = new textfuzzy("cats running ran cactus cactuses community communities");
		$this->assertEqual($fuzzy->getstemmed(), "cat run ran cactus cactus commune commune");

		$fuzzy = new textfuzzy("stemming algorithms understanding working cases different corrections its");
		$this->assertEqual($fuzzy->getstemmed(), "stem algorithm understand work case differ correct it");

		$fuzzy = new textfuzzy("disciplines bibliometric measures western countries evident cultures");
		$this->assertEqual($fuzzy->getstemmed(), "discipline bibliometry measure west country evidence culture");

		$fuzzy = new textfuzzy("scientific classified sciences phenomena experimented specialized expertise");
		$this->assertEqual($fuzzy->getstemmed(), "science classify science phenomenon experiment special expert");

		$fuzzy = new textfuzzy("ideologies notably democracy politics incorporated socialists principles principals formerly");
		$this->assertEqual($fuzzy->getstemmed(), "ideology notable democrat politic incorporate social principle principal former");

		$fuzzy = new textfuzzy("errors approaches positions pieces occurrences halves quarters grows backtracking");
		$this->assertEqual($fuzzy->getstemmed(), "error approach position piece occur half quarter grow backtrack");

		$fuzzy = new textfuzzy("engaged measures documentation installation institutions marketing promotion activities");
		$this->assertEqual($fuzzy->getstemmed(), "engage measure document install institute market promote activity");

		$fuzzy = new textfuzzy("demonstrably essential additional percentage presentation developed synchronisation");
		$this->assertEqual($fuzzy->getstemmed(), "demonstrate essence add percent present develop synchronise");

		$a = "the traditional research focus on the number of pokies per capita and per suburb and their proximity to people's homes misses the human factors he says.  Instead the swinburne project has applied a much broader definition of access incorporating psychological as well as social and demographic variables.  It's all very good to be talking about the existence of ATMs within gambling venues or the number of machines per capita in an area - they are all good indicators of some aspects of access, but we think that it actually misses the more human element which can be I'm lonely, it's midnight, what are my options?' he says.";
		$b = "the tradition research focus on the number of pokie per capital and per suburb and their proximity to people home miss the human factor he say instead the swinburne project has apply a much broad define of access incorporate psychology as well as social and demograph vary it all very good to be talk about the exist of atms with gamble venue or the number of machine per capital in an area they are all good indicator of some aspect of access but we think that it actual miss the more human element which can be im lone it midnight what are my option he say";

		$fuzzy = new textfuzzy($a);
		$this->assertEqual($fuzzy->getstemmed(), $b);

		// echo "\n" . $b . "\n" . $fuzzy->getstemmed() . "\n";

		$fuzzy = new textfuzzy("funny boxes tables farms houses mites months dewdrops");
		$this->assertEqual($fuzzy->getstemmed(), "fun box table farm house mite month dewdrop");
	}
}

?>
